#!/usr/bin/python
#backup_rebuilder.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import time
import random


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in backup_rebuilder.py')

from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet import threads


import lib.dhnio as dhnio
import lib.misc as misc
import lib.nameurl as nameurl
import lib.transport_control as transport_control
import lib.settings as settings
import lib.contacts as contacts
import lib.eccmap as eccmap
import lib.tmpfile as tmpfile
import lib.diskspace as diskspace
import lib.packetid as packetid
from lib.automat import Automat


import fire_hire
import backup_monitor
import block_rebuilder
import data_sender
import lib.automats as automats

import backup_matrix
import raidread
import io_throttle

_BackupRebuilder = None
_StoppedFlag = True
_BackupIDsQueue = []  
_BlockRebuildersQueue = []

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _BackupRebuilder
    if _BackupRebuilder is None:
        _BackupRebuilder = BackupRebuilder('backup_rebuilder', 'STOPPED', 12)
    if event is not None:
        _BackupRebuilder.automat(event, arg)
    return _BackupRebuilder


class BackupRebuilder(Automat):
    timers = {
        'timer-1min':   (2*60, ['REQUEST']),
        'timer-10sec':  (10.0, ['REQUEST']),
        'timer-1sec':   (1.0, ['NEXT_BACKUP', 'REQUEST']),
        }
    
    def init(self):
        self.currentBackupID = None             # currently working on this backup
        self.currentBlockNumber = -1            # currently working on this block
        self.workingBlocksQueue = []            # list of missing blocks we work on for current backup
        self.missingPackets = 0 
        self.missingSuppliers = set()

    def state_changed(self, oldstate, newstate):
        # automats.set_global_state('REBUILD ' + newstate)
        backup_monitor.A('backup_rebuilder.state', newstate)
        
    def A(self, event, arg):
        #---REQUEST---
        if self.state is 'REQUEST':
            if event == 'timer-1sec' and self.isStopped(arg) :
                self.state = 'STOPPED'
            elif ( event == 'timer-10sec' or event == 'inbox-data-packet' or event == 'requests-sent' ) and self.isChanceToRebuild(arg) :
                self.state = 'REBUILDING'
                self.doAttemptRebuild(arg)
            elif event == 'timer-1min' or ( event == 'requests-sent' and self.isRequestQueueEmpty(arg) and not self.isMissingPackets(arg) ) :
                self.state = 'DONE'
        #---STOPPED---
        elif self.state is 'STOPPED':
            if event == 'init' :
                pass
            elif event == 'start' :
                self.state = 'NEXT_BACKUP'
                self.doClearStoppedFlag(arg)
        #---NEXT_BACKUP---
        elif self.state is 'NEXT_BACKUP':
            if event == 'timer-1sec' and not self.isStopped(arg) and self.isMoreBackups(arg) :
                self.state = 'PREPARE'
                self.doPrepareNextBackup(arg)
            elif event == 'timer-1sec' and not self.isMoreBackups(arg) and not self.isStopped(arg) :
                self.state = 'DONE'
            elif event == 'timer-1sec' and self.isStopped(arg) :
                self.state = 'STOPPED'
        #---DONE---
        elif self.state is 'DONE':
            if event == 'start' :
                self.state = 'NEXT_BACKUP'
                self.doClearStoppedFlag(arg)
        #---PREPARE---
        elif self.state is 'PREPARE':
            if event == 'backup-ready' and self.isStopped(arg) :
                self.state = 'STOPPED'
            elif event == 'backup-ready' and not self.isStopped(arg) and self.isMoreBlocks(arg) :
                self.state = 'REQUEST'
                self.doRequestAvailableBlocks(arg)
            elif event == 'backup-ready' and not self.isStopped(arg) and not self.isMoreBlocks(arg) and self.isMoreBackups(arg) :
                self.state = 'NEXT_BACKUP'
            elif event == 'backup-ready' and ( not self.isMoreBackups(arg) and not self.isMoreBlocks(arg) ) :
                self.state = 'DONE'
        #---REBUILDING---
        elif self.state is 'REBUILDING':
            if event == 'rebuilding-finished' and self.isStopped(arg) :
                self.state = 'STOPPED'
            elif event == 'rebuilding-finished' and not self.isStopped(arg) and self.isMoreBlocks(arg) :
                self.state = 'REQUEST'
                self.doRequestAvailableBlocks(arg)
            elif event == 'rebuilding-finished' and not self.isStopped(arg) and not self.isMoreBlocks(arg) :
                self.state = 'PREPARE'
                self.doPrepareNextBackup(arg)

    def isMoreBackups(self, arg):
        global _BackupIDsQueue
        return len(_BackupIDsQueue) > 0
    
    def isMoreBlocks(self, arg):
        # because started from 0,  -1 means not found
        return len(self.workingBlocksQueue) > 0 
        # return self.currentBlockNumber > -1
        
    def isMissingPackets(self, arg):
        return self.missingPackets > 0 

    def isStopped(self, arg):
        return ReadStoppedFlag() == True # :-)
    
    def isChanceToRebuild(self, arg):
#         return len(self.missingSuppliers) <= eccmap.Current().CorrectableErrors
        supplierSet = backup_matrix.suppliers_set()
        # start checking in reverse order, see below for explanation
        for blockIndex in range(len(self.workingBlocksQueue)-1, -1, -1):
            blockNumber = self.workingBlocksQueue[blockIndex]
            if eccmap.Current().CanMakeProgress(
                    backup_matrix.GetLocalDataArray(self.currentBackupID, blockNumber),
                    backup_matrix.GetLocalParityArray(self.currentBackupID, blockNumber)):
                return True
        return False
    
    def isRequestQueueEmpty(self, arg):
        supplierSet = backup_matrix.suppliers_set()
        for supplierNum in range(supplierSet.supplierCount):
            supplierID = supplierSet.suppliers[supplierNum]
            if io_throttle.HasBackupIDInSendQueue(supplierID, self.currentBackupID):
                return False
        return True

    def doPrepareNextBackup(self, arg):
        global _BackupIDsQueue
        # clear block number from previous iteration
        self.currentBlockNumber = -1
        # check it, may be we already fixed all things
        if len(_BackupIDsQueue) == 0:
            self.workingBlocksQueue = []
            self.automat('backup-ready')
            return
        # take a first backup from queue to work on it
        backupID = _BackupIDsQueue.pop(0)
        # if remote data structure is not exist for this backup - create it
        # this mean this is only local backup!
        if not backup_matrix.remote_files().has_key(backupID):
            backup_matrix.remote_files()[backupID] = {}
            # we create empty remote info for every local block
            # range(0) should return []
            for blockNum in range(backup_matrix.local_max_block_numbers().get(backupID, -1) + 1):
                backup_matrix.remote_files()[backupID][blockNum] = {
                    'D': [0] * backup_matrix.suppliers_set().supplierCount,
                    'P': [0] * backup_matrix.suppliers_set().supplierCount }
        # detect missing blocks from remote info
        self.workingBlocksQueue = backup_matrix.ScanMissingBlocks(backupID)
        dhnio.Dprint(8, 'backup_rebuilder.doPrepareNextBackup [%s] working blocks: %s' % (backupID, str(self.workingBlocksQueue)))
        # find the correct max block number for this backup
        # we can have remote and local files
        # will take biggest block number from both 
        backupMaxBlock = max(backup_matrix.remote_max_block_numbers().get(backupID, -1),
                             backup_matrix.local_max_block_numbers().get(backupID, -1))
        # now need to remember this biggest block number
        # remote info may have less blocks - need to create empty info for missing blocks
        for blockNum in range(backupMaxBlock + 1):
            if backup_matrix.remote_files()[backupID].has_key(blockNum):
                continue
            backup_matrix.remote_files()[backupID][blockNum] = {
                'D': [0] * backup_matrix.suppliers_set().supplierCount,
                'P': [0] * backup_matrix.suppliers_set().supplierCount }
        if self.currentBackupID:
            # clear requesting queue from previous task
            io_throttle.DeleteBackupRequests(self.currentBackupID)
        # really take the next backup
        self.currentBackupID = backupID
        # clear requesting queue, remove old packets for this backup, we will send them again
        io_throttle.DeleteBackupRequests(self.currentBackupID)
        # dhnio.Dprint(6, 'backup_rebuilder.doTakeNextBackup currentBackupID=%s workingBlocksQueue=%d' % (self.currentBackupID, len(self.workingBlocksQueue)))
        self.automat('backup-ready')

    def doRequestAvailableBlocks(self, arg):
        self.missingPackets = 0
        # self.missingSuppliers.clear()
        # here we want to request some packets before we start working to rebuild the missed blocks
        supplierSet = backup_matrix.suppliers_set()
        availableSuppliers = supplierSet.GetActiveArray()
        # remember how many requests we did on this iteration
        total_requests_count = 0
        # at the moment I do download everything I have available and needed
        if '' in contacts.getSupplierIDs():
            self.automat('requests-sent', total_requests_count)
            return
        for supplierNum in range(supplierSet.supplierCount):
            supplierID = supplierSet.suppliers[supplierNum]
            requests_count = 0
            # we do requests in reverse order because we start rebuilding from the last block 
            # for blockNum in range(self.currentBlockNumber, -1, -1):
            for blockIndex in range(len(self.workingBlocksQueue)-1, -1, -1):
                blockNum = self.workingBlocksQueue[blockIndex] 
                # do not keep too many requests in the queue
                if io_throttle.GetRequestQueueLength(supplierID) >= 16:
                    break
                # also don't do too many requests at once
                if requests_count > 16:
                    break
                remoteData = backup_matrix.GetRemoteDataArray(self.currentBackupID, blockNum)
                remoteParity = backup_matrix.GetRemoteParityArray(self.currentBackupID, blockNum)
                localData = backup_matrix.GetLocalDataArray(self.currentBackupID, blockNum)
                localParity = backup_matrix.GetLocalParityArray(self.currentBackupID, blockNum)
                # if the remote Data exist and is available because supplier is on line,
                # but we do not have it on hand - do request  
                if localData[supplierNum] == 0:
                    PacketID = packetid.MakePacketID(self.currentBackupID, blockNum, supplierNum, 'Data')
                    if remoteData[supplierNum] == 1:
                        if availableSuppliers[supplierNum]:
                            # if supplier is not alive - we can't request from him           
                            if not io_throttle.HasPacketInRequestQueue(supplierID, PacketID):
                                io_throttle.QueueRequestFile(
                                    self.FileReceived, 
                                    misc.getLocalID(), 
                                    PacketID, 
                                    misc.getLocalID(), 
                                    supplierID)
                                requests_count += 1
                    else:
                        # count this packet as missing
                        self.missingPackets += 1
                        # also mark this guy as one who dont have any data - nor local nor remote 
                        # self.missingSuppliers.add(supplierNum)
                # same for Parity
                if localParity[supplierNum] == 0:
                    PacketID = packetid.MakePacketID(self.currentBackupID, blockNum, supplierNum, 'Parity')
                    if remoteParity[supplierNum] == 1: 
                        if availableSuppliers[supplierNum]:
                            if not io_throttle.HasPacketInRequestQueue(supplierID, PacketID):
                                io_throttle.QueueRequestFile(
                                    self.FileReceived, 
                                    misc.getLocalID(), 
                                    PacketID, 
                                    misc.getLocalID(), 
                                    supplierID)
                                requests_count += 1
                    else:
                        self.missingPackets += 1
                        # self.missingSuppliers.add(supplierNum)
            total_requests_count += requests_count
        self.automat('requests-sent', total_requests_count)
                
    def doAttemptRebuild(self, arg):
        self.workBlock = None
        self.blocksSucceed = []
        if len(self.workingBlocksQueue) == 0:
            self.automat('rebuilding-finished', False)
            return            
        # let's rebuild the backup blocks in reverse order, take last blocks first ... 
        # in such way we can propagate how big is the whole backup as soon as possible!
        # remote machine can multiply [file size] * [block number] 
        # and calculate the whole size to be received ... smart!
        # ... remote supplier should not use last file to calculate
        self.blockIndex = len(self.workingBlocksQueue) - 1
        dhnio.Dprint(8, 'backup_rebuilder.doAttemptRebuild %d more blocks' % (self.blockIndex+1))
        def _prepare_one_block(): 
            if self.blockIndex < 0:
                # dhnio.Dprint(8, '        _prepare_one_block finish all blocks')
                reactor.callLater(0, _finish_all_blocks)
                return
            self.currentBlockNumber = self.workingBlocksQueue[self.blockIndex]
            # dhnio.Dprint(8, '        _prepare_one_block %d to rebuild' % self.currentBlockNumber)
            self.workBlock = block_rebuilder.BlockRebuilder(
                eccmap.Current(), #self.eccMap,
                self.currentBackupID,
                self.currentBlockNumber,
                backup_matrix.suppliers_set(),
                backup_matrix.GetRemoteDataArray(self.currentBackupID, self.currentBlockNumber),
                backup_matrix.GetRemoteParityArray(self.currentBackupID, self.currentBlockNumber),
                backup_matrix.GetLocalDataArray(self.currentBackupID, self.currentBlockNumber),
                backup_matrix.GetLocalParityArray(self.currentBackupID, self.currentBlockNumber),)
            reactor.callLater(0, _identify_block_packets)
        def _identify_block_packets():
            self.workBlock.IdentifyMissing()
#            if not self.workBlock.IsMissingFilesOnHand():
#                dhnio.Dprint(8, '        _identify_block_packets some missing files is not come yet')
#                reactor.callLater(0, self.automat, 'rebuilding-finished', False)
#                return
            reactor.callLater(0, _work_on_block)
        def _work_on_block():
            maybeDeferred(self.workBlock.AttemptRebuild).addCallback(_rebuild_finished)
        def _rebuild_finished(someNewData):
            # dhnio.Dprint(8, '        _rebuild_finished on block %d, result is %s' % (self.currentBlockNumber, str(someNewData)))
            if someNewData:
                self.workBlock.WorkDoneReport()
                self.blocksSucceed.append(self.currentBlockNumber)
                data_sender.A('new-data')
            self.workBlock = None
            self.blockIndex -= 1
            reactor.callLater(0, _prepare_one_block)
        def _finish_all_blocks():
            for blockNum in self.blocksSucceed:
                self.workingBlocksQueue.remove(blockNum)
            dhnio.Dprint(8, 'backup_rebuilder.doAttemptRebuild._finish_all_blocks succeed:%s working:%s' % (str(self.blocksSucceed), str(self.workingBlocksQueue)))
            result = len(self.blocksSucceed) > 0
            self.blocksSucceed = []
            self.automat('rebuilding-finished', result)
        reactor.callLater(0, _prepare_one_block)

    def doClearStoppedFlag(self, arg):
        ClearStoppedFlag()

    def FileReceived(self, packet, state):
        if state in ['in queue', 'shutdown', 'exist']:
            return
        if state != 'received':
            dhnio.Dprint(4, "backup_rebuilder.FileReceived WARNING incorrect state [%s] for packet %s" % (str(state), str(packet)))
            return
        packetID = packet.PacketID
        filename = os.path.join(settings.getLocalBackupsDir(), packetID)
        if packet.Valid():
            # if we managed to rebuild a file 
            # before a request came in don't overwrite it
            if os.path.exists(filename):
                dhnio.Dprint(4, "backup_rebuilder.FileReceived WARNING rewriting existed file" + filename)
                try: 
                    os.remove(filename)
                except:
                    dhnio.DprintException()
            dhnio.WriteFile(filename, packet.Payload)
        else:
            # TODO 
            # if we didn't get a valid packet ... re-request it or delete it?
            dhnio.Dprint(2, "backup_rebuilder.FileReceived WARNING " + filename + " is not a valid packet, remove it now")
            try: 
                os.remove(filename)
            except:
                dhnio.DprintException()
        backup_matrix.LocalFileReport(packetID)
        self.automat('inbox-data-packet', packetID)
        
#------------------------------------------------------------------------------ 

def AddBackupsToWork(backupIDs):
    global _BackupIDsQueue 
    _BackupIDsQueue.extend(backupIDs)


def RemoveBackupToWork(backupID):
    global _BackupIDsQueue
    if backupID in _BackupIDsQueue:
        _BackupIDsQueue.remove(backupID)


def SetStoppedFlag():
    global _StoppedFlag
    _StoppedFlag = True
    
    
def ClearStoppedFlag():
    global _StoppedFlag
    _StoppedFlag = False
    

def ReadStoppedFlag():
    global _StoppedFlag
    return _StoppedFlag
    
    
#def AddBlockRebuilder(obj):
#    global _BlockRebuildersQueue
#    dhnio.Dprint(10, 'backup_rebuilder.AddBlockRebuilder for %s-%s' % (obj.backupID, str(obj.blockNum)))
#    _BlockRebuildersQueue.append(obj)
    

#def RemoveBlockRebuilder(obj):
#    global _BlockRebuildersQueue
#    dhnio.Dprint(10, 'backup_rebuilder.RemoveBlockRebuilder for %s-%s' % (obj.backupID, str(obj.blockNum)))
#    _BlockRebuildersQueue.remove(obj)


    
    

    



#!/usr/bin/python
#raidread.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

#
# This is to read some possibly incomplete raid data and recover original data.
#
# Basic idea is to find a parity that is only missing one data, then fix that
#  data, and look at parities again to see if another is now missing only one.
#  This can be called an "iterative algorithm" or "belief propagation".
#  There are algorithms that can recover data when this can not, but
#    they are more complicated.
# Input is a file and a RAID spec.
# The spec says for each datablock which parity blocks XOR it.
# As we read in each datablock we XOR all the parity blocks for it.
# Probably best to read in all the datablocks at once, and do all
# the parities one time.  If we did one datablock after another
# the parity blocks would need to be active all the time.
#
# If we XOR integers, we have a byte order issue probably, though
# maybe not since they all stay the same order, whatever that is.
#
# Some machines are 64-bit.  Would be fun to make use of that if
# we have it.
#
# 2^1 => 3       ^ is XOR operator  - so maybe we can do it in Python
#
# In the production code, when there are multiple errors we really want
# to fix the one we can fix fastest first.  The only danger is that
# we get too many errors at the same time.  We reduce this danger if
# we fix the one we can fix fastest first.


import struct
import os
import sys


import lib.dhnio as dhnio
import lib.eccmap as eccmap
import lib.settings as settings

INTSIZE = 4

#------------------------------------------------------------------------------ 

_ECCMAP = {}
def geteccmap(name):
    global _ECCMAP
    if not _ECCMAP.has_key(name):
        _ECCMAP[name] = eccmap.eccmap(name) 
    return _ECCMAP[name]

#------------------------------------------------------------------------------ 

def shutdown():
    global _ECCMAP
    _ECCMAP.clear()

#------------------------------------------------------------------------------ 

#RebuildOne_new and RebuildOne_orig are just for debugging purposes
def RebuildOne(inlist, listlen, outfilename):
    readsize = 1 # vary from 1 byte to 4 bytes
    raidfiles = [''] * listlen # range(listlen)   # just need a list of this size
    raidreads = [''] * listlen # range(listlen)
    for filenum in xrange(listlen):
        try:
            raidfiles[filenum] = open(inlist[filenum], "rb")
        except:
            dhnio.DprintException()
            for f in raidfiles:
                try:
                    f.close()
                except:
                    pass 
            return 
    rebuildfile = open(outfilename, "wb")
    loopcount = 0
    debugLoopcount = 2000
    debugI = 2000
    while True:
        for k in xrange(listlen):
            raidreads[k] = raidfiles[k].read(2048)
        if not raidreads[0]:
            break
        i = 0
        while i < len(raidreads[0]):
            xor = 0
            for j in xrange(listlen):
                b1 = ord(raidreads[j][i])
                xor = xor ^ b1
            rebuildfile.write(chr(xor))
            i += readsize
    for filenum in xrange(listlen):
        raidfiles[filenum].close()
    rebuildfile.close()


def RebuildOne_new(inlist, listlen, outfilename):
    # INTSIZE = settings.IntSize()
    fds = range(0,listlen)   # just need a list of this size
    wholefile = dhnio.ReadBinaryFile(inlist[0])
    seglength = len(wholefile)   # just needed length of file
    for filenum in xrange(listlen):
        fds[filenum]=open(inlist[filenum],"r")
    fout = open(outfilename,"w")
    for i in xrange(seglength):
        xor = 0
        for j in xrange(listlen):
            b1 = ord(fds[j].read(1))
            xor = xor ^ b1
        fout.write(chr(xor))
    for filenum in xrange(listlen):
        fds[filenum].close

# We XOR list of listlen input files and write result to a file named outfilename
def RebuildOne_orig(inlist, listlen, outfilename):
        # INTSIZE = settings.IntSize()
        fds = range(0,listlen)   # just need a list of this size
        wholefile = dhnio.ReadBinaryFile(inlist[0])
        seglength=len(wholefile)   # just needed length of file
        for filenum in range(0, listlen):
                fds[filenum]=open(inlist[filenum],"rb")
        fout=open(outfilename,"w")
        for i in range(0,seglength/INTSIZE):
                xor=0
                for j in range(0, listlen):
                        bstr1 = fds[j].read(INTSIZE)
                        b1, = struct.unpack(">l", bstr1)
                        xor = xor ^ b1
                outstr = struct.pack(">l", xor)
                fout.write(outstr)
        for filenum in range(0, listlen):
                fds[filenum].close

# If segment is good, there is a file for it, if not then no file exists.
# We only rebuild data segments.
# Could only make parity segments from existing data segments, so no help toward getting data.
# As long as we could rebuild one more data segment in a pass,
#  we do another pass to see if we are then able to do another.
#  When we can't do anything more we could how many good data segments there
#    are, and if we have all we win, if not we fail.

def raidread(OutputFileName, eccmapname, backupId, blockNumber, data_parity_dir=None):
    if data_parity_dir is None:
        data_parity_dir = settings.getLocalBackupsDir()

    # INTSIZE = settings.IntSize()
    # myeccmap = eccmap.eccmap(eccmapname)
    myeccmap = geteccmap(eccmapname)
    GoodFiles = range(0, 200)
    MakingProgress = 1
    while MakingProgress == 1:
        MakingProgress = 0
        for PSegNum in xrange(myeccmap.paritysegments):
                PFileName = os.path.join(data_parity_dir, backupId,  str(blockNumber) + '-' + str(PSegNum) + '-Parity')
                if os.path.exists(PFileName):
                        Map = myeccmap.ParityToData[PSegNum]
                        TotalDSegs = 0
                        GoodDSegs = 0
                        for DSegNum in Map:
                                TotalDSegs+=1
                                FileName = os.path.join(data_parity_dir, backupId, str(blockNumber) + '-' + str(DSegNum) + '-Data')
                                if os.path.exists(FileName):
                                        GoodFiles[GoodDSegs]=FileName
                                        GoodDSegs+=1
                                else:
                                        BadName=FileName
                        if GoodDSegs == TotalDSegs - 1:
                                MakingProgress = 1
                                GoodFiles[GoodDSegs]=PFileName
                                GoodDSegs += 1
                                RebuildOne(GoodFiles,GoodDSegs,BadName)

    #  Count up the good segments and combine
    GoodDSegs = 0
    output = open(OutputFileName, "wb")
    for DSegNum in xrange(myeccmap.datasegments):
        FileName = os.path.join(data_parity_dir, backupId, str(blockNumber) + '-' + str(DSegNum) + '-Data')
        if os.path.exists(FileName):
            GoodDSegs += 1
            moredata = open(FileName,"rb").read()
            output.write(moredata)
    output.close()
    return GoodDSegs


def main():
    if (len(sys.argv) < 3):
        print "raidread needs an output filename and eccmap name"
        sys.exit(2)

    OutputFileName = sys.argv[1]
    eccmapname = sys.argv[2]

    print "raidread is starting with"
    print OutputFileName
    print eccmapname
    raidread(OutputFileName, eccmapname)


if __name__ == "__main__":
    main()





#!/usr/bin/python
#message.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
#  This is sort of email that has solved the spam problem.  Could spread to general messaging use.
#
#  Users have list of identities that can send them messages without paying ("correspondents" list or "friends and family").
#
#  User specifies postage price for incoming mail delivery from someone not on their list in identity.
#    If incoming mail is not on ID list and does not have enough payment, it is dropped.  Spam gone.
#    If incoming mail needs to be paid for we look and see if that ID has a high enough balance.
#
#  In identity.py we need to have spam-price published.   Ok to put IDURL on web pages because
#  can't get junk mail without making money.
#
#  Message body is always encrypted with key of destination
#
#  We learn of payments when we get deposit reciept from either sender or accounts.datahaven.net.
#     We credit that contact.
#
#  When message comes in we check if they have enough credit or if they are on the correspondent list.
#     If so we:
#             send it to guimessage
#             send back an ack letting sender know user can look at it
#
#  Can also have a blacklist where we take there money and still toss the message.
#
#  Bit worried about the potential for a customer or supplier to threaten a contact.  Probably
#     need to have GUI explain that contacts from customers/suppliers are not normal and
#     anything strange should be reason to fire that customer/supplier right away.
#
#  Contacts needs to have 3 types now.  And possible for some contact to be 2 or 3 types.  The
#  types are:  suppliers, customers, correspondents
#
#  Will make message.py   first then guimessage.


import os
import sys
import datetime
import time
import StringIO


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in message.py')


from twisted.internet.defer import Deferred


import lib.dhnio as dhnio
import lib.misc as misc
import lib.dhnpacket as dhnpacket
import lib.contacts as contacts
import lib.commands as commands
##import lib.eccmap as eccmap
import lib.settings as settings
import lib.transport_control as transport_control
import lib.identity as identity
import lib.identitycache as identitycache
import lib.packetid as packetid
import lib.dhncrypto as dhncrypto
import lib.nameurl as nameurl

OnIncommingMessageFunc = None

#------------------------------------------------------------------------------ 


def init():
    dhnio.Dprint(4,"message.init")
##    guimessage.UpdateCorrespondents()

def ConnectCorrespondent(idurl):
    pass

def UniqueID():
    return str(int(time.time()*100.0))

# We always encrypt messages with a session key
# so we need to package with encrypted body
class MessageClass:
    def __init__(self, destinationidentity, messagebody):
        dhnio.Dprint(8, "message.MessageClass making message ")
        sessionkey = dhncrypto.NewSessionKey()
        keystring = destinationidentity.publickey
        self.encryptedKey = dhncrypto.EncryptStringPK(keystring, sessionkey)
        self.encryptedMessage = dhncrypto.EncryptWithSessionKey(sessionkey, messagebody)
        
    def ClearBody(self):
        sessionkey = dhncrypto.DecryptLocalPK(self.encryptedKey)
        # we only decrypt with LocalIdentity
        return dhncrypto.DecryptWithSessionKey(sessionkey, self.encryptedMessage)

# Message came in for us so we:
#   1) Check that it is a correspondent
#   2) Decrypt message body
#   3) Call messagegui
def Message(request):
    global OnIncommingMessageFunc
    dhnio.Dprint(6, "message.Message from " + str(request.OwnerID))
    senderidentity = contacts.getCorrespondent(request.OwnerID)
    if not senderidentity:
        dhnio.Dprint(4,"message.Message WARNING had sender not in correspondents list " + request.OwnerID)
        return
    Amessage = misc.StringToObject(request.Payload)
    if Amessage is None:
        dhnio.Dprint(4,"message.Message WARNING wrong Payload, can not extract message from request")
        return
    clearmessage = Amessage.ClearBody()
    SaveMessage(clearmessage)
    if OnIncommingMessageFunc is not None:
        OnIncommingMessageFunc(request, SplitMessage(clearmessage))
    transport_control.SendAck(request)

def MakeMessage(to, subj, body, dt=datetime.datetime.now().strftime("%Y/%m/%d %I:%M:%S %p")):
    dhnio.Dprint(6, "message.MakeMessage to " + to)
    msg = ( misc.getLocalID(),
            to,
            subj,
            dt,
            body)
    return '\n'.join(msg)

def SplitMessage(clearmessage):
    fin = StringIO.StringIO(clearmessage)
    sender = fin.readline().strip()
    to = fin.readline().strip()
    subject = fin.readline().strip()
    datetime = fin.readline().strip()
    body = fin.read()
    return (sender, to, subject, datetime, body)

def SaveMessage(clearmessage):
    msguid = UniqueID()
    dhnio.Dprint(6, "message.SaveMessage %s" % msguid)
    msgfilename = os.path.join(settings.getMessagesDir(),  msguid+'.dhnmessage')
    msgfile = file(msgfilename, 'w')
    msgfile.write(str(clearmessage))
    msgfile.close()
    return msguid

def LoadMessage(msgpath):
    fin = open(msgpath, 'r')
    clearmessage = fin.read()
    fin.close()
    return SplitMessage(clearmessage) 

def ReadMessage(messageuid):
    msgpath = os.path.join(settings.getMessagesDir(), messageuid + '.dhnmessage')
    # msgpath = settings.getMessagesDir() + os.sep + messageuid + '.dhnmessage'
    if not os.path.exists(msgpath):
        return None
    return LoadMessage(msgpath)

def DeleteMessage(messageuid):
    msgpath = os.path.join(settings.getMessagesDir(), messageuid + '.dhnmessage')
    if not os.path.exists(msgpath):
        return False
    try:
        os.remove(msgpath)
    except:
        dhnio.DprintException()
        return False
    dhnio.Dprint(6, "message.DeleteMessage %s" % messageuid)
    return True

# PREPRO - we should probably check that we have a recent copy of the remote identity in identity cache
# or else transport_control needs to be able to fetch identities again when it needs to do more than X retries
# guimessage calls this
def SendMessage(RemoteID, messagebody, PacketID=""):
    dhnio.Dprint(6, "message.SendMessage to: " + str(RemoteID) )
    #TODO ERROR HERE (return Defer)
    if not identitycache.scheduleForCaching(RemoteID):
        dhnio.Dprint(1, "message.SendMessage ERROR. Can't find identity: " + str(RemoteID))
        return
    RemoteIdentity=identitycache.FromCache(RemoteID)
    if RemoteIdentity == '':
        dhnio.Dprint(1, "message.SendMessage ERROR. Can't retreive identity: " + str(RemoteID))
        return
    Amessage = MessageClass(RemoteIdentity, messagebody)
    MyID = misc.getLocalID()
    if PacketID == "":
        PacketID = packetid.UniqueID()
    Payload = misc.ObjectToString(Amessage)
    dhnio.Dprint(6, "message.SendMessage  about to send to " + RemoteID)
    result = dhnpacket.dhnpacket(commands.Message(),  MyID, MyID, PacketID, Payload, RemoteID)
    transport_control.outboxAck(result)

def ListAllMessages():
    mlist = []
    i = 0
    for filename in os.listdir(settings.getMessagesDir()):
        if not filename.endswith('.dhnmessage'):
            continue
        msgpath = os.path.join(settings.getMessagesDir(), filename)
        # msgpath = settings.getMessagesDir() + os.sep + filename
        if not os.path.exists(msgpath):
            continue
        msg = LoadMessage(msgpath)
        messageuid = filename.split('.')[0]
        msgtupple = (messageuid, nameurl.GetName(msg[0]), nameurl.GetName(msg[1]), msg[3], msg[2])
        mlist.append(msgtupple)
        i += 1
    return mlist

def SortMessagesList(mlist, sort_by_column):
    order = {}
    i = 0
    for msg in mlist:
        order[msg[sort_by_column]] = i
        i += 1
    keys = order.keys()
    keys.sort()
    sorted = []
    for key in keys:
        sorted.append(order[key])
    return sorted

#------------------------------------------------------------------------------ 


if __name__ == "__main__":
    init()
    reactor.run()




#!/usr/bin/python
#webtraffic.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in trafficstats.py')

from twisted.web import server, resource


#-------------------------------------------------------------------------------


#(total bytes, finished packets, failed packets, total packets)
_InboxPacketsCount = 0
_InboxByIDURL = {}
_InboxByHost = {}
_InboxByProto = {}
_InboxByType = {}

_OutboxPacketsCount = 0
_OutboxByIDURL = {}
_OutboxByHost = {}
_OutboxByProto = {}
_OutboxByType = {}

_WebListener = None

_DefaultReloadTimeout = 600

#-------------------------------------------------------------------------------

def init(root=None, path='traffic', port=9997):
    global _WebListener
    if root is not None:
        root.putChild(path, TrafficPage())
        return
    if _WebListener:
        return
    root = resource.Resource()
    root.putChild('', TrafficPage())
    site = server.Site(root)
    try:
        _WebListener = reactor.listenTCP(port, site)
    except:
        pass

def shutdown():
    global _WebListener
    if _WebListener:
        _WebListener.stopListening()
        del _WebListener
        _WebListener = None

def inbox_packets_count():
    global _InboxPacketsCount
    return _InboxPacketsCount
    
def inbox_by_idurl():
    global _InboxByIDURL
    return _InboxByIDURL
    
def inbox_by_host():
    global _InboxByHost
    return _InboxByHost
    
def inbox_by_proto():
    global _InboxByProto
    return _InboxByProto

def inbox_by_type():
    global _InboxByType
    return _InboxByType
    
def outbox_packets_count():
    global _OutboxPacketsCount
    return _OutboxPacketsCount
    
def outbox_by_idurl():
    global _OutboxByIDURL
    return _OutboxByIDURL
    
def outbox_by_host():
    global _OutboxByHost
    return _OutboxByHost
    
def outbox_by_proto(): 
    global _OutboxByProto
    return _OutboxByProto

def outbox_by_type():
    global _OutboxByType
    return _OutboxByType

def inbox(newpacket, proto, host_, status):
    global _InboxPacketsCount
    global _InboxByIDURL
    global _InboxByHost
    global _InboxByProto
    global _InboxByType
    
    if newpacket is None:
        return

    bytes = len(newpacket)
    idurl = newpacket.CreatorID
    host = '%s://%s' % (proto, host_)
    type = newpacket.Command

    if not _InboxByIDURL.has_key(idurl):
        _InboxByIDURL[idurl] = [0, 0, 0, 0]
    _InboxByIDURL[idurl][0] += bytes
    if status == 'finished':
        _InboxByIDURL[idurl][1] += 1
    else:
        _InboxByIDURL[idurl][2] += 1
    _InboxByIDURL[idurl][3] += 1

    if not _InboxByHost.has_key(host):
        _InboxByHost[host] = [0, 0, 0, 0]
    _InboxByHost[host][0] += bytes
    if status == 'finished':
        _InboxByHost[host][1] += 1
    else:
        _InboxByHost[host][2] += 1
    _InboxByHost[host][3] += 1

    if not _InboxByProto.has_key(proto):
        _InboxByProto[proto] = [0, 0, 0, 0]
    _InboxByProto[proto][0] += bytes
    if status == 'finished':
        _InboxByProto[proto][1] += 1
    else:
        _InboxByProto[proto][2] += 1
    _InboxByProto[proto][3] += 1
    
    if not _InboxByType.has_key(type):
        _InboxByType[type] = [0, 0, 0, 0]
    _InboxByType[type][0] += bytes
    if status == 'finished':
        _InboxByType[type][1] += 1
    else:
        _InboxByType[type][2] += 1
    _InboxByType[type][3] += 1

    _InboxPacketsCount += 1

def outbox(workitem, proto, host_, status):
    global _OutboxPacketsCount
    global _OutboxByIDURL
    global _OutboxByHost
    global _OutboxByProto
    global _OutboxByType

    bytes = workitem.filesize
    idurl = workitem.remoteid
    host = '%s://%s' % (proto, host_)
    type = workitem.command

    if not _OutboxByIDURL.has_key(idurl):
        _OutboxByIDURL[idurl] = [0, 0, 0, 0]
    _OutboxByIDURL[idurl][0] += bytes
    if status == 'finished':
        _OutboxByIDURL[idurl][1] += 1
    else:
        _OutboxByIDURL[idurl][2] += 1
    _OutboxByIDURL[idurl][3] += 1

    if not _OutboxByHost.has_key(host):
        _OutboxByHost[host] = [0, 0, 0, 0]
    _OutboxByHost[host][0] += bytes
    if status == 'finished':
        _OutboxByHost[host][1] += 1
    else:
        _OutboxByHost[host][2] += 1
    _OutboxByHost[host][3] += 1

    if not _OutboxByProto.has_key(proto):
        _OutboxByProto[proto] = [0, 0, 0, 0]
    _OutboxByProto[proto][0] += bytes
    if status == 'finished':
        _OutboxByProto[proto][1] += 1
    else:
        _OutboxByProto[proto][2] += 1
    _OutboxByProto[proto][3] += 1

    if not _OutboxByType.has_key(type):
        _OutboxByType[type] = [0, 0, 0, 0]
    _OutboxByType[type][0] += bytes
    if status == 'finished':
        _OutboxByType[type][1] += 1
    else:
        _OutboxByType[type][2] += 1
    _OutboxByType[type][3] += 1

    _OutboxPacketsCount += 1

#-------------------------------------------------------------------------------

class TrafficPage(resource.Resource):
    header_html = '''<html><head>
<meta http-equiv="refresh" content="%(reload)s">
<title>Traffic</title></head>
<body bgcolor="#FFFFFF" text="#000000" link="#0000FF" vlink="#0000FF">
<form action="%(baseurl)s?" method="get">
<input size="4" name="reload" value="%(reload)s" />
<input type="submit" value="update" />
</form>
<a href="%(baseurl)s?reload=1&type=%(type)s&dir=%(dir)s">[1 sec.]</a>|
<a href="%(baseurl)s?reload=5&type=%(type)s&dir=%(dir)s">[5 sec.]</a>|
<a href="%(baseurl)s?reload=10&type=%(type)s&dir=%(dir)s">[10 sec.]</a>|
<a href="%(baseurl)s?reload=60&type=%(type)s&dir=%(dir)s">[60 sec.]</a>
<br>
<a href="%(baseurl)s?type=idurl&reload=%(reload)s&dir=%(dir)s">[by idurl]</a>|
<a href="%(baseurl)s?type=host&reload=%(reload)s&dir=%(dir)s">[by host]</a>|
<a href="%(baseurl)s?type=proto&reload=%(reload)s&dir=%(dir)s">[by proto]</a>
<a href="%(baseurl)s?type=type&reload=%(reload)s&dir=%(dir)s">[by type]</a>
<br>
<a href="%(baseurl)s?type=%(type)s&reload=%(reload)s&dir=in">[income traffic]</a>|
<a href="%(baseurl)s?type=%(type)s&reload=%(reload)s&dir=out">[outgoing traffic]</a>
<br>
<table><tr>
<td align=left>%(type)s
<td>total bytes
<td>total packets
<td>finished packets
<td>failed packets
'''

    def render(self, request):
        global _InboxPacketsCount
        global _InboxByIDURL
        global _InboxByHost
        global _InboxByProto
        global _OutboxPacketsCount
        global _OutboxByIDURL
        global _OutboxByHost
        global _OutboxByProto

        direction = request.args.get('dir', [''])[0]
        if direction not in ('in', 'out'):
            direction = 'in'
        typ = request.args.get('type', [''])[0]
        if typ not in ('idurl', 'host', 'proto', 'type'):
            typ = 'idurl'
        reloadS = request.args.get('reload', [''])[0]
        try:
            reloadV = int(reloadS)
        except:
            reloadV = _DefaultReloadTimeout

        d = {'type': typ, 'reload': str(reloadV), 'dir': direction, 'baseurl': request.path}
        out = self.header_html % d
        if direction == 'in':
            if typ == 'idurl':
                for i, v in _InboxByIDURL.items():
                    out += '<tr><td><a href="%s">%s</a><td>%d<td>%d<td>%d<td>%d\n' % (
                        i, i, v[0], v[3], v[1], v[2])
            elif typ == 'host':
                for i, v in _InboxByHost.items():
                    out += '<tr><td>%s<td>%d<td>%d<td>%d<td>%d\n' % (
                        i, v[0], v[3], v[1], v[2])
            elif typ == 'proto':
                for i, v in _InboxByProto.items():
                    out += '<tr><td>%s<td>%d<td>%d<td>%d<td>%d\n' % (
                        i, v[0], v[3], v[1], v[2])
            elif typ == 'type':
                for i, v in _InboxByType.items():
                    out += '<tr><td>%s<td>%d<td>%d<td>%d<td>%d\n' % (
                        i, v[0], v[3], v[1], v[2])
        else:
            if typ == 'idurl':
                for i, v in _OutboxByIDURL.items():
                    out += '<tr><td><a href="%s">%s</a><td>%d<td>%d<td>%d<td>%d\n' % (
                        i, i, v[0], v[3], v[1], v[2])
            elif typ == 'host':
                for i, v in _OutboxByHost.items():
                    out += '<tr><td>%s<td>%d<td>%d<td>%d<td>%d\n' % (
                        i, v[0], v[3], v[1], v[2])
            elif typ == 'proto':
                for i, v in _OutboxByProto.items():
                    out += '<tr><td>%s<td>%d<td>%d<td>%d<td>%d\n' % (
                        i, v[0], v[3], v[1], v[2])
            elif typ == 'type':
                for i, v in _OutboxByType.items():
                    out += '<tr><td>%s<td>%d<td>%d<td>%d<td>%d\n' % (
                        i, v[0], v[3], v[1], v[2])

        out += '</table>'
        if direction == 'in':
            out += '<p>total income packets: %d</p>' % _InboxPacketsCount
        else:
            out += '<p>total outgoing packets: %d</p>' % _OutboxPacketsCount
        out += '</body></html>'

        return out

#------------------------------------------------------------------------------ 

if __name__ == "__main__":
    init()
    reactor.run()




from nitro.async import AsyncOp
from nitro.bencode import encode, decode, DecodeError

from cspace.util.validate import validateIPAddress, validateInetAddress

class RoutedLocation( object ) :
    def __init__( self, routerAddr, routerId ) :
        self.routerAddr = routerAddr
        self.routerId = routerId
    
    def __cmp__( self, other ) :
        if other is None :
            return 1
        return cmp( (self.routerAddr,self.routerId), (other.routerAddr,other.routerId) )

class DirectLocation( object ) :
    def __init__( self, addr ) :
        self.addr = addr

    def __cmp__( self, other ) :
        if other is None :
            return 1
        return cmp( self.addr, other.addr )

class UserLocation( object ) :
    def __init__( self, directLocations, routedLocations, publicIP ) :
        self.directLocations = directLocations
        self.routedLocations = routedLocations
        self.publicIP = publicIP

    def __cmp__( self, other ) :
        if other is None :
            return 1
        selfKey = ( self.directLocations, self.routedLocations,
                self.publicIP )
        otherKey = ( other.directLocations, other.routedLocations,
                other.publicIP )
        return cmp( selfKey, otherKey )

    def encode( self ) :
        directList = [loc.addr for loc in self.directLocations]
        routedList = [(loc.routerAddr,loc.routerId) for loc in self.routedLocations]
        data = { 'direct' : directList,
            'routed' : routedList,
            'publicIP' : self.publicIP }
        return encode( data )

    @staticmethod
    def decode( data ) :
        data = decode( data )
        if type(data) is not dict : raise TypeError
        publicIP = data.get( 'publicIP' )
        try :
            validateIPAddress( publicIP )
        except :
            publicIP = ''
        directLocations = []
        routedLocations = []
        for direct in data.get('direct',[]) :
            if type(direct) is not list : continue
            direct = tuple( direct )
            try :
                validateInetAddress( direct )
            except :
                continue
            directLocations.append( DirectLocation(direct) )
        for routed in data.get('routed',[]) :
            if type(routed) is not list : continue
            if len(routed) != 2 : continue
            routerAddr,routerId = routed
            if type(routerId) is not str : continue
            if type(routerAddr) is not list : continue
            routerAddr = tuple( routerAddr )
            try :
                validateInetAddress( routerAddr )
            except :
                continue
            routedLocations.append( RoutedLocation(routerAddr,routerId) )
        return UserLocation( directLocations, routedLocations, publicIP )

def publishUserLocation( rsaKey, location, updateLevel, dhtClient,
        nodeTable, callback=None ) :
    def onResult( putCount, updateLevel ) :
        op.notify( putCount>0, updateLevel )
    dhtOp = dhtClient.lookupPutKey( rsaKey, location.encode(),
            updateLevel, nodeTable.getLiveNodes(), onResult )
    op = AsyncOp( callback, dhtOp.cancel )
    return op

def lookupUser( publicKey, dhtClient, nodeTable, callback=None ) :
    def onResult( result ) :
        if len(result) == 0 :
            op.notify( None )
            return
        try :
            location = UserLocation.decode( result[0][0] )
        except :
            op.notify( None )
            return
        op.notify( location )
    dhtOp = dhtClient.lookupGetKey( publicKey, nodeTable.getLiveNodes(),
            onResult )
    op = AsyncOp( callback, dhtOp.cancel )
    return op




# Code for NATs and the like. Also includes code for determining local IP
# address (suprisingly tricky, in the presence of STUPID STUPID STUPID
# networking stacks)

from twisted.internet import defer
from twisted.internet.protocol import DatagramProtocol
import random, socket
from twisted.python import log
from defcache import DeferredCache

# from shtoom.interfaces import StunPolicy as IStunPolicy

_Debug = False

class LocalNetworkMulticast(DatagramProtocol, object):

    def __init__(self, *args, **kwargs):
        self.compDef = defer.Deferred()
        self.completed = False
        super(LocalNetworkMulticast,self).__init__(*args, **kwargs)

    def listenMulticast(self):
        from twisted.internet import reactor
        from twisted.internet.error import CannotListenError
        attempt = 0
        port = 11000 + random.randint(0,5000)
        while True:
            try:
                mcast = reactor.listenMulticast(port, self)
                break
            except CannotListenError:
                port = 11000 + random.randint(0,5000)
                attempt += 1
                if _Debug:
                    print "listenmulticast failed, trying", port
        if attempt > 5:
            log.msg("warning: couldn't listen ony mcast port", system='network')
            d, self.compDef = self.compDef, None
            d.callback(None)
        dd = mcast.joinGroup('239.255.255.250', socket.INADDR_ANY)
        def _cb(x):
            if _Debug:
                print 'multicast group joined'
        def _eb(x):
            if _Debug:
                print 'multicast group join failed', x
        dd.addCallback(_cb)
        dd.addErrback(_eb)
        self.mcastPort = port

    def blatMCast(self):
        try:
            # XXX might need to set an option to make sure we see our own packets
            self.transport.write('ping', ('239.255.255.250', self.mcastPort))
            self.transport.write('ping', ('239.255.255.250', self.mcastPort))
            self.transport.write('ping', ('239.255.255.250', self.mcastPort))
        except:
            pass

    def datagramReceived(self, dgram, addr):
        if self.completed:
            return
        elif dgram != 'ping':
            return
        else:
            self.completed = True
            d, self.compDef = self.compDef, None
            d.callback(addr[0])

_cachedLocalIP = None
def _cacheLocalIP(res):
    global _cachedLocalIP
    if _Debug: print "caching value", res
    _cachedLocalIP = res
    return res

# If there's a need to clear the cache, call this method (e.g. DHCP client)
def _clearCachedLocalIP():
    _cacheLocalIP(None)

def _getLocalIPAddress():
    # So much pain. Don't even bother with
    # socket.gethostbyname(socket.gethostname()) - the number of ways this
    # is broken is beyond belief.
    from twisted.internet import reactor
    global _cachedLocalIP
    if _Debug:
        print '_getLocalIPAddress', _cachedLocalIP
    if _cachedLocalIP is not None:
        return defer.succeed(_cachedLocalIP)
    # first we try a connected udp socket
    if _Debug: 
        print "resolving A.ROOT-SERVERS.NET"
    d = reactor.resolve('A.ROOT-SERVERS.NET')
    d.addCallbacks(_getLocalIPAddressViaConnectedUDP, _noDNSerrback)
    return d

getLocalIPAddress = DeferredCache(_getLocalIPAddress)

# def clearCache():
#     "Clear cached NAT settings (e.g. when moving to a different network)"
#     from shtoom.upnp import clearCache as uClearCache
#     from shtoom.stun import clearCache as sClearCache
#     print "clearing all NAT caches"
#     getLocalIPAddress.clearCache()
#     getMapper.clearCache()
#     uClearCache()
#     sClearCache()

def _noDNSerrback(failure):
    # No global DNS? What the heck, it's possible, I guess.
    if _Debug: 
        print "no DNS, trying multicast"
    return _getLocalIPAddressViaMulticast()

def _getLocalIPAddressViaConnectedUDP(ip):
    from twisted.internet import reactor
    from twisted.internet.protocol import DatagramProtocol
    if _Debug: 
        print "connecting UDP socket to", ip
    prot = DatagramProtocol()
    p = reactor.listenUDP(0, prot)
    try:
        res = prot.transport.connect(ip, 7)
    except:
        if _Debug: 
            print "can not connect to %s:%d" % ( ip, 7 )
        return _getLocalIPAddressViaMulticast()
    locip = prot.transport.getHost().host
    p.stopListening()
    del prot, p
    if _Debug: 
        print "connected UDP socket says", locip
    if isBogusAddress(locip):
        # #$#*(&??!@#$!!!
        if _Debug: 
            print "connected UDP socket gives crack, trying mcast instead"
        return _getLocalIPAddressViaMulticast()
    else:
        return locip


def _getLocalIPAddressViaMulticast():
    # We listen on a new multicast address (using UPnP group, and
    # a random port) and send out a packet to that address - we get
    # our own packet back and get the address from it.
    from twisted.internet import reactor
    from twisted.internet.interfaces import IReactorMulticast
    try:
        IReactorMulticast(reactor)
    except:
        if _Debug: 
            print "no multicast support in reactor"
        log.msg("warning: no multicast in reactor", system='network')
        return None
    locprot = LocalNetworkMulticast()
    if _Debug: print "listening to multicast"
    locprot.listenMulticast()
    if _Debug: print "sending multicast packets"
    locprot.blatMCast()
    locprot.compDef.addCallback(_cacheLocalIP)
    return locprot.compDef

# def cb_detectNAT(res):
#     (ufired,upnp), (sfired,stun) = res
#     if not ufired and not sfired:
#         log.msg("no STUN or UPnP results", system="nat")
#         return None
#     if ufired:
#         return upnp
#     return stun
#
# def detectNAT():
#     # We prefer UPnP when available, as it's less pissing about (ha!)
#     from shtoom.upnp import getUPnP
#     from shtoom.stun import getSTUN
#     ud = getUPnP()
#     sd = getSTUN()
#     dl = defer.DeferredList([ud, sd])
#     dl.addCallback(cb_detectNAT).addErrback(log.err)
#     return dl
#
# def cb_getMapper(res):
#     from shtoom.upnp import getMapper as getUMapper
#     from shtoom.stun import getMapper as getSTUNMapper
#     (ufired,upnp), (sfired,stun) = res
#     log.msg("detectNAT got %r"%res, system="nat")
#     if not upnp and not stun:
#         log.msg("no STUN or UPnP results", system="nat")
#         return getNullMapper()
#     if upnp:
#         log.msg("using UPnP mapper", system="nat")
#         return getUMapper()
#     if stun.useful:
#         log.msg("using STUN mapper", system="nat")
#         return getSTUNMapper()
#     log.msg("No UPnP, and STUN is useless", system="nat")
#     return getNullMapper()
#
# _forcedMapper = None
#
# _installedShutdownHook = False
# def getMapper():
#     # We prefer UPnP when available, as it's more robust
#     global _installedShutdownHook
#     if not _installedShutdownHook:
#         from twisted.internet import reactor
#         t = reactor.addSystemEventTrigger('after',
#                                           'shutdown',
#                                           clearCache)
#         _installedShutdownHook = True
#     try:
#         from __main__ import app
#     except:
#         app = None
#     natPref = 'both'
#     if app is not None:
#         print "app is", app
#         natPref = app.getPref('nat')
#         log.msg('NAT preference says to use %s'%(natPref))
#     if _forcedMapper is not None:
#         return defer.succeed(_forcedMapper)
#     from shtoom.upnp import getUPnP
#     from shtoom.stun import getSTUN
#     if natPref == 'both':
#         ud = getUPnP()
#         sd = getSTUN()
#         d = defer.DeferredList([ud, sd])
#     elif natPref == 'upnp':
#         ud = getUPnP()
#         d = defer.DeferredList([ud, defer.succeed(None) ])
#     elif natPref == 'stun':
#         ud = getSTUN()
#         d = defer.DeferredList([defer.succeed(None), sd])
#     else:
#         nm = NullMapper()
#         d = defer.DeferredList([defer.succeed(None),
#                                 defer.succeed(None)])
#     d.addCallback(cb_getMapper).addErrback(log.err)
#     return d
# getMapper = DeferredCache(getMapper, inProgressOnly=False)
#
# def _forceMapper(mapper):
#     global _forcedMapper
#     _forcedMapper = mapper

def isBogusAddress(addr):
    """ Returns true if the given address is bogus, i.e. 0.0.0.0 or
        127.0.0.1. Additional forms of bogus might be added later.
    """
    if addr.startswith('0.') or addr.startswith('127.'):
        return True
    return False

# class BaseMapper:
#     "Base class with useful functionality for Mappers"
#     _ptypes = []
#
#     def _checkValidPort(self, port):
#         from twisted.internet.base import BasePort
#         # Ugh. Why is there no IPort ?
#         if not isinstance(port, BasePort):
#             raise ValueError("expected a Port, got %r"%(port))
#         # XXX Check it's listening! How???
#         if not hasattr(port, 'socket'):
#             raise ValueError("Port %r appears to be closed"%(port))
#
#         locAddr = port.getHost()
#         if locAddr.type not in self._ptypes:
#             raise ValueError("can only map %s, not %s"%
#                         (', '.join(self._ptypes),locAddr.type))
#         if locAddr.port == 0:
#             raise ValueError("Port %r has port number of 0"%(port))
#
#         if not port.connected:
#             raise ValueError("Port %r is not listening"%(port))
#
# class NullMapper(BaseMapper):
#     "Mapper that does nothing"
#
#     _ptypes = ( 'TCP', 'UDP' )
#
#     def __init__(self):
#         self._mapped = {}
#
#     def map(self, port):
#         "See shtoom.interfaces.NATMapper.map"
#         self._checkValidPort(port)
#         if port in self._mapped:
#             return defer.succeed(self._mapped[port])
#         cd = defer.Deferred()
#         self._mapped[port] = cd
#         locAddr = port.getHost().host
#         if isBogusAddress(locAddr):
#             # lookup local IP.
#             d = getLocalIPAddress()
#             d.addCallback(lambda x: self._cb_map_gotLocalIP(x, port))
#         else:
#             reactor.callLater(0, lambda: self._cb_map_gotLocalIP(locAddr, port))
#         return cd
#     map = DeferredCache(map, inProgressOnly=True)
#
#     def _cb_map_gotLocalIP(self, locIP, port):
#         cd = self._mapped[port]
#         self._mapped[port] = (locIP, port.getHost().port)
#         cd.callback(self._mapped[port])
#
#     def info(self, port):
#         "See shtoom.interfaces.NATMapper.info"
#         if port in self._mapped:
#             return self._mapped[port]
#         else:
#             raise ValueError('Port %r is not currently mapped'%(port))
#
#     def unmap(self, port):
#         "See shtoom.interfaces.NATMapper.unmap"
#         # A no-op for NullMapper
#         if port not in self._mapped:
#             raise ValueError('Port %r is not currently mapped'%(port))
#         del self._mapped[port]
#         return defer.succeed(None)
#
# _cached_nullmapper = None
# def getNullMapper():
#     global _cached_nullmapper
#     if _cached_nullmapper is None:
#         _cached_nullmapper = NullMapper()
#     return _cached_nullmapper
#
# class NetAddress:
#     """ A class that represents a net address of the form
#         foo/nbits, e.g. 10/8, or 192.168/16, or whatever
#     """
#     def __init__(self, netaddress):
#         parts = netaddress.split('/')
#         if len(parts) > 2:
#             raise ValueError, "should be of form address/mask"
#         if len(parts) == 1:
#             ip, mask = parts[0], 32
#         else:
#             ip, mask = parts[0], int(parts[1])
#         if mask < 0 or mask > 32:
#             raise ValueError, "mask should be between 0 and 32"
#
#         self.net = self.inet_aton(ip)
#         self.mask = ( 2L**32 -1 ) ^ ( 2L**(32-mask) - 1 )
#         self.start = self.net
#         self.end = self.start | (2L**(32-mask) - 1)
#
#     def inet_aton(self, ipstr):
#         "A sane inet_aton"
#         if ':' in ipstr:
#             return
#         net = [ int(x) for x in ipstr.split('.') ] + [ 0,0,0 ]
#         net = net[:4]
#         return  ((((((0L+net[0])<<8) + net[1])<<8) + net[2])<<8) +net[3]
#
#     def inet_ntoa(self, ip):
#         import socket, struct
#         return socket.inet_ntoa(struct.pack('!I',ip))
#
#     def __repr__(self):
#         return '<NetAddress %s/%s (%s-%s) at %#x>'%(self.inet_ntoa(self.net),
#                                            self.inet_ntoa(self.mask),
#                                            self.inet_ntoa(self.start),
#                                            self.inet_ntoa(self.end),
#                                            id(self))
#
#     def check(self, ip):
#         "Check if an IP or network is contained in this network address"
#         if isinstance(ip, NetAddress):
#             return self.check(ip.start) and self.check(ip.end)
#         if isinstance(ip, basestring):
#             ip = self.inet_aton(ip)
#         if ip is None:
#             return False
#         if ip & self.mask == self.net:
#             return True
#         else:
#             return False
#
#     __contains__ = check
#
#
# class AlwaysStun:
#     __implements__ = IStunPolicy
#
#     def checkStun(self, localip, remoteip):
#         return True
#
# class NeverStun:
#     __implements__ = IStunPolicy
#
#     def checkStun(self, localip, remoteip):
#         return False
#
# class RFC1918Stun:
#     "A sane default policy"
#     __implements__ = IStunPolicy
#
#     addresses = ( NetAddress('10/8'),
#                   NetAddress('172.16/12'),
#                   NetAddress('192.168/16'),
#                   NetAddress('127/8') )
#     localhost = NetAddress('127/8')
#
#     def checkStun(self, localip, remoteip):
#         localIsRFC1918 = False
#         remoteIsRFC1918 = False
#         remoteIsLocalhost = False
#         # Yay. getPeer() returns a name, not an IP
#         #  XXX tofix: grab radix's goodns.py until it
#         # lands in twisted proper.
#         # Until then, use this getaddrinfo() hack.
#         if not remoteip:
#             return None
#         if remoteip[0] not in '0123456789':
#             import socket
#             try:
#                 ai = socket.getaddrinfo(remoteip, None)
#             except (socket.error, socket.gaierror):
#                 return None
#             remoteips = [x[4][0] for x in ai]
#         else:
#             remoteips = [remoteip,]
#         for net in self.addresses:
#             if localip in net:
#                 localIsRFC1918 = True
#             # See comments above. Worse, if the host has an address that's
#             # RFC1918, and externally advertised (which is wrong, and broken),
#             # the STUN check will be incorrect. Bah.
#             for remoteip in remoteips:
#                 if remoteip in net:
#                     remoteIsRFC1918 = True
#                 if remoteip in self.localhost:
#                     remoteIsLocalhost = True
#         if localIsRFC1918 and not (remoteIsRFC1918 or remoteIsLocalhost):
#             return True
#         else:
#             return False
#
# _defaultPolicy = RFC1918Stun()
# def installPolicy(policy):
#     global _defaultPolicy
#     _defaultPolicy = policy
#
# def getPolicy():
#     return _defaultPolicy



if __name__ == "__main__":
#     from twisted.internet import gtk2reactor
#     gtk2reactor.install()
    from twisted.internet import reactor
    import sys

    log.FileLogObserver.timeFormat = "%H:%M:%S"
    log.startLogging(sys.stdout)

    def cb_gotip(addr):
        print "got local IP address of", addr
#     def cb_gotnat(res):
#         print "got NAT of", res
    d1 = getLocalIPAddress().addCallback(cb_gotip)
#     d2 = detectNAT().addCallback(cb_gotnat)
#     dl = defer.DeferredList([d1,d2])
#     dl.addCallback(lambda x:reactor.stop())
    d1.addCallback(lambda x:reactor.stop())
    reactor.run()



from nitro.tcp import TCPStream

class TCPLineStream( object ) :
    def __init__( self, sock, reactor ) :
        self.reactor = reactor
        self.stream = TCPStream( sock, reactor )
        self.readEnabled = False
        self.readInitiated = False
        self.timerOp = None
        self.inputBuffer = [ '' ]
        self.inputCallback = None
        self.stream.setInputCallback( self._onInput )

        self.getSock = self.stream.getSock
        self.setCloseCallback = self.stream.setCloseCallback
        self.setErrorCallback = self.stream.setErrorCallback
        self.setWriteCompleteCallback = self.stream.setWriteCompleteCallback
        self.hasShutdown = self.stream.hasShutdown
        self.writeData = self.stream.writeData

    def setInputCallback( self, inputCallback ) :
        self.inputCallback = inputCallback

    def shutdown( self ) :
        self.enableRead( False )
        self.stream.shutdown()

    def close( self, deferred=False, callback=None ) :
        self.enableRead( False )
        return self.stream.close( deferred, callback )

    def enableRead( self, enable=True ) :
        assert not self.hasShutdown()
        if enable != self.readEnabled :
            self.readEnabled = enable
            if enable :
                assert not self.readInitiated
                assert not self.timerOp
                if len(self.inputBuffer) > 1 :
                    self.timerOp = self.reactor.addTimer( 0, self._onTimer )
                else :
                    self.readInitiated = True
                    self.stream.initiateRead( 4096 )
            else :
                if self.timerOp :
                    self.timerOp.cancel()
                    self.timerOp = None
                else :
                    self.readInitiated = False
                    self.stream.cancelRead()

    def _onInput( self, data ) :
        while data :
            i = data.find( '\n' )
            if i < 0 :
                self.inputBuffer[-1] += data
                break
            else :
                i += 1
                self.inputBuffer[-1] += data[:i]
                self.inputBuffer.append( '' )
                data = data[i:]
        while len(self.inputBuffer) > 1 :
            line = self.inputBuffer.pop( 0 )
            self.inputCallback( line )
            if not self.readEnabled : return

    def _onTimer( self ) :
        while len(self.inputBuffer) > 1 :
            line = self.inputBuffer.pop( 0 )
            self.inputCallback( line )
            if not self.readEnabled : return
        if self.timerOp :
            self.timerOp.cancel()
            self.timerOp = None
            self.readInitiated = True
            self.stream.initiateRead( 4096 )



#!/usr/bin/python
#misc.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#


import os
import sys
import random
import time
import math
import hmac
import hashlib
import base64
import string
import subprocess
import re
import tempfile
import urllib
import locale


from twisted.spread import banana
from twisted.spread import jelly
from twisted.python.win32 import cmdLineQuote
from twisted.internet.defer import fail

import nonblocking
import dhnio
import settings
import identity
import dhnnet
import tmpfile
import dhnpacket
import packetid


#if __name__ == '__main__':
    #dirpath = os.path.dirname(os.path.abspath(__file__))
    #sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..', '..')))
    #print '\n'.join(sys.path)
    #sys.modules['datahaven.lib.dhnpacket'] = dhnpacket
    #sys.modules['datahaven.lib.dhnpacket'].dhnpacket = str #dhnpacket.dhnpacket
    #import datahaven.lib.dhnpacket as dhnpacket


#-------------------------------------------------------------------------------


_RemoveAfterSent = True

# Functions outside should use getLocalIdentity()
_LocalIdentity = None
_LocalIDURL = None
_LocalName = None

# if we come up with more valid transports,
# we'll need to add them here
# in the case we don't have any valid transports,
# we use this array as the default
# order is important!
# this is default order to be used for new users
# more stable transports must be higher
validTransports = [ 'http', 'cspace', 'q2q', 'tcp', 'ssh', 'email', 'skype', 'udp']

_AttenuationFactor = 1.1

#-------------------------------------------------------------------------------

def init():
    dhnio.Dprint(4, 'misc.init')

#-------------------------------------------------------------------------------

def isLocalIdentityReady():
    global _LocalIdentity
    return _LocalIdentity is not None

def setLocalIdentity(ident):
    global _LocalIdentity
    global _LocalIDURL
    global _LocalName
    _LocalIdentity = ident
    _LocalIDURL = _LocalIdentity.getIDURL()
    _LocalName = _LocalIdentity.getIDName()

def setLocalIdentityXML(idxml):
    global _LocalIdentity
    _LocalIdentity = identity.identity(xmlsrc=idxml)

def getLocalIdentity():
    """return my identity object"""
    global _LocalIdentity
    if not isLocalIdentityReady():
        loadLocalIdentity()
    return _LocalIdentity

def getLocalID():
    """return my IDURL""" 
    global _LocalIDURL
    if _LocalIDURL is None:
        localIdent = getLocalIdentity()
        if localIdent:
            _LocalIDURL = localIdent.getIDURL()
    return _LocalIDURL

def getIDName():
    """return my account name (without '.xml')"""
    global _LocalName
    if _LocalName is None:
        if isLocalIdentityReady():
            _LocalName = getLocalIdentity().getIDName()
    return _LocalName

def loadLocalIdentity():
    global _LocalIdentity
    global _LocalIDURL
    global _LocalName
    xmlid = ''
    filename = settings.LocalIdentityFilename()
    if os.path.exists(filename):
        xmlid = dhnio.ReadTextFile(filename)
        dhnio.Dprint(6, 'misc.loadLocalIdentity %d bytes read from %s' % (len(xmlid), filename))
    if xmlid == '':
        dhnio.Dprint(2, "misc.loadLocalIdentity ERROR reading local identity from " + filename)
        return
#        dhnio.Dprint(4, 'misc.loadLocalIdentity loading failed. Making default Identity')
#        ident = identity.makeDefaultIdentity()
#        xmlid = ident.serialize()
#        dhnio.WriteFile(filename, xmlid)
    lid = identity.identity(xmlsrc=xmlid)
    if not lid.Valid():
        dhnio.Dprint(2, "misc.loadLocalIdentity ERROR local identity is not Valid")
        return
    _LocalIdentity = lid
    _LocalIDURL = lid.getIDURL()
    _LocalName = lid.getIDName()
    setTransportOrder(getOrderFromContacts(_LocalIdentity))
    dhnio.Dprint(6, "misc.loadLocalIdentity my name is [%s]" % lid.getIDName())

def saveLocalIdentity():
    global _LocalIdentity
    if not isLocalIdentityReady():
        dhnio.Dprint(2, "misc.saveLocalIdentity ERROR localidentity not exist!")
        return
    dhnio.Dprint(6, "misc.saveLocalIdentity")
    _LocalIdentity.sign()
    xmlid = _LocalIdentity.serialize()
    filename = settings.LocalIdentityFilename()
    dhnio.WriteFile(filename, xmlid)

#------------------------------------------------------------------------------ 

def readLocalIP():
    return dhnio.ReadBinaryFile(settings.LocalIPFilename())

def readExternalIP():
    return dhnio.ReadBinaryFile(settings.ExternalIPFilename())

def readSupplierData(idurl, filename):
    path = settings.SupplierPath(idurl, filename)
    if not os.path.isfile(path):
        return ''
    return dhnio.ReadTextFile(path)

def writeSupplierData(idurl, filename, data):
    dirPath = settings.SupplierPath(idurl)
    if not os.path.isdir(dirPath):
        os.makedirs(dirPath)
    path = settings.SupplierPath(idurl, filename)
    return dhnio.WriteFile(path, data)

#-------------------------------------------------------------------------------

# let's try to make backupID as a filepath
# and keep files on remote suppliers into directory tree.
# so we will have something like remote-distributed file system
# some links to learn
# http://en.wikipedia.org/wiki/Filename 
# http://nedbatchelder.com/blog/201106/filenames_with_accents.html
# http://wiki.apache.org/subversion/NonNormalizingUnicodeCompositionAwareness
def NewBackupID(time_st=None):
    if time_st is None:
        time_st = time.localtime()
    ampm = time.strftime("%p", time_st)
    if ampm == '':
        dhnio.Dprint(2, 'misc.NewBackupID WARNING time.strftime("%p") returns empty string')
        ampm = 'AM' if time.time() % 86400 < 43200 else 'PM'
    result = "F" + time.strftime("%Y%m%d%I%M%S", time_st) + ampm
    # dhnio.Dprint(10, 'misc.NewBackupID ' + result)
    return result

def TimeFromBackupID(backupID):
    try:
        if backupID.endswith('AM') or backupID.endswith('PM'):
            ampm = backupID[-2:]
            st_time = list(time.strptime(backupID[1:-2], '%Y%m%d%I%M%S'))
        else:
            i = backupID.rfind('M')
            ampm = backupID[i-1:i+1]
            st_time = list(time.strptime(backupID[1:i-1], '%Y%m%d%I%M%S'))
        if ampm == 'PM':
            st_time[3] += 12
        return time.mktime(st_time)
    except:
        dhnio.DprintException()
        return None

# next functions are to come up with a sorted list of backup ids (dealing with AM/PM)
def modified_version(a):
    try:
        if a.endswith('AM') or a.endswith('PM'):
            int_a = int(a[1:-2])
            int_b = 0
        else:
            i = a.rfind('M')
            int_a = int(a[1:i-1])
            int_b = int(a[i+1:])
    except:
        dhnio.DprintException()
        return -1  
    hour = a[-8:-6]
    if a.endswith('PM') and hour != '12':
        int_a += 120000
    elif a.endswith('AM') and hour == '12':
        int_a -= 120000
    return int_a + int_b

def version_compare(version1, version2):
    return cmp(modified_version(version1), modified_version(version2))

def backup_id_compare(backupID1, backupID2):
    if isinstance(backupID1, tuple):
        backupID1 = backupID1[0]
        backupID2 = backupID2[0]
    pathID1, version1 = packetid.SplitBackupID(backupID1)
    pathID2, version2 = packetid.SplitBackupID(backupID2)
    if pathID1 is None or pathID2 is None:
        return 0
    if pathID1 != pathID2:
        return cmp(pathID1, pathID2)
    return version_compare(version1, version2)

def sorted_backup_ids(backupIds, reverse=False):
    sorted_ids = sorted(backupIds, backup_id_compare, reverse=reverse)
#    if reverse:
#        sorted_ids.reverse()
    return sorted_ids

def sorted_versions(versions, reverse=False):
    sorted_versions_list = sorted(versions, version_compare, reverse=reverse)
#    if reverse:
#        sorted_versions_list.reverse()
    return sorted_versions_list

#------------------------------------------------------------------------------ 

BASE_ALPHABET = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!()+-#$^&_=@[]{}`~' 
BASE_LIST = BASE_ALPHABET
BASE_DICT = dict((c, i) for i, c in enumerate(BASE_LIST))

def base_decode(string, reverse_base=BASE_DICT):
    return base64.decodestring(string)
#    length = len(reverse_base)
#    ret = 0
#    for i, c in enumerate(string[::-1]):
#        ret += (length ** i) * reverse_base[c]
#    return ret

def base_encode(string, base=BASE_LIST):
    return base64.encodestring(string)
#    length = len(base)
#    ret = ''
#    while integer != 0:
#        ret = base[integer % length] + ret
#        integer /= length
#    return ret

def FilePathToBackupID(filepath):
    # be sure the string is in unicode
    fp = dhnio.portablePath(filepath)
    # now convert to string with char codes as numbers
    result = ''
    for part in fp.split('/'):
        if part == '':
            result += '/'
            continue
        if len(result) > 0:
            # skip first separator
            # if this is Linux absolute path  
            result += '/'
        # switch from unicode if it was
        part = str(part) # unicode(part).encode()
        # word = base64.b64encode(part).replace('+', '-').replace('/', '_')
        # word = base64.b64encode(hashlib.sha1(part+os.urandom(8)).digest()).replace('+', '-').replace('/', '_').replace('=', '')
        # word = hashlib.sha1(part+os.urandom(8)).hexdigest()
        print base64.b64encode(os.urandom(6))
        word = hashlib.md5(part+os.urandom(4)).hexdigest()
        print len(word)
#        # compress the string
#        if compress:
#            # import zlib
#            # word = base_encode(zlib.compress(upart, 9))
#            word = base_encode(upart)
#        else:
#            word = u''
#            for c in upart:
#                word += '%04d' % ord(c)
#        word = word.strip()
        # print '[%s]' % word
        result += word 
    return result

def BackupIDToFilePath(backupID, decompress=False):
    result = ''
    part = ''
    ch = ''
    for c in backupID:
        if c == '/':
            if part != '':
                # base64.b64encode(part).replace('+', '-').replace('/', '_')
                result += base64.b64decode(part.replace('-', '+').replace('_', '/'))
#                if decompress:
#                    import zlib
#                    # result += zlib.decompress(base_decode(part))
#                    result += base_decode(part)
#                else:
#                    result += part
            result += c
            part = ''
        else:
            part += c
#            if decompress:
#                part += c
#            else:
#                ch += c
#                if len(ch) == 4:
#                    try:
#                        v = int(ch)
#                    except:
#                        dhnio.DprintException()
#                    part += unichr(v)
#                    ch = ''
    if part:
        result += base64.b64decode(part.replace('-', '+').replace('_', '/'))
        # result += part
#        if decompress:
#            import zlib
#            result += zlib.decompress(base_decode(part))
#        else:
#            result += part
    # we return string in unicode
    return unicode(result)

#------------------------------------------------------------------------------ 

def DigitsOnly(input, includes=''):
    return ''.join([c for c in input if c in '0123456789' + includes])

def ToInt(input, default=0):
    try:
        return int(input)
    except:
        return default
    
def ToFloat(input, default=0.0):
    try:
        return float(input)
    except:
        return default

def IsDigitsOnly(input):
    for c in input:
        if c not in '0123456789':
            return False
    return result

#------------------------------------------------------------------------------ 

def ValidUserName(username):
    if len(username) < settings.MinimumUsernameLength():
        return False
    if len(username) > settings.MaximumUsernameLength():
        return False
    for c in username:
        if c not in settings.LegalUsernameChars():
            return False
    if username[0] not in set('abcdefghijklmnopqrstuvwxyz'):
        return False
    if username.startswith('dhn'):
        return False
    return True

def ValidEmail(email, full_check=True):
    regexp = '^[\w\-\.\@]*$'
    if re.match(regexp,email) == None:
        return False
    if email.startswith('.'):
        return False
    if email.endswith('.'):
        return False
    if email.startswith('-'):
        return False
    if email.endswith('-'):
        return False
    if email.startswith('@'):
        return False
    if email.endswith('@'):
        return False
    if len(email) < 3:
        return False
    if full_check:
        regexp2 = '^[\w\-\.]*\@[\w\-\.]*$'
        if re.match(regexp2,email) == None:
            return False
    return True

def ValidPhone(value):
    regexp = '^[ \d\-\+]*$'
    if re.match(regexp,value) == None:
        return False
    if len(value) < 5:
        return False
    return True

def ValidName(value):
    regexp = '^[\w\-]*$'
    if re.match(regexp, value) == None:
        return False
    if len(value) > 100:
        return False
    return True

def MakeValidHTMLComment(text):
    ret = ''
    for c in text:
        if c in set("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-+*/=_()[]{}:;,.?!@#$%|~ "):
            ret += c
    return ret

def ValidateBitCoinAddress(strAddr):
    """ does simple validation of a bitcoin address. 
    param : strAddr : an ASCII or unicode string, of a bitcoin public address.
    returns : boolean, indicating that the address has a correct format.
    http://www.rugatu.com/questions/3255/anybody-has-python-code-to-verifyvalidate-bitcoin-address"""
    # The first character indicates the "version" of the address.
    CHARS_OK_FIRST = "123"
    # alphanumeric characters without : l I O 0
    CHARS_OK = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
    # We do not check the high length limit of the adress. 
    if len(strAddr) < 27:
        return False
    if len(strAddr) > 35:
        return False
    if strAddr[0] not in CHARS_OK_FIRST:
        return False
    # We use the function "all" by passing it an enumerator as parameter. 
    # It does a little optimisation : 
    # if one of the character is not valid, the next ones are not tested.
    return all( ( char in CHARS_OK for char in strAddr[1:] ) )

#------------------------------------------------------------------------------ 

# For some things we need to have files which are round sizes
# for example some encryption needs files that are multiples of 8 bytes
# This function rounds filename up to the next multiple of stepsize
def RoundupFile(filename, stepsize):
    try:
        size = os.path.getsize(filename)
    except:
        return
    mod = size % stepsize;
    increase=0;
    if mod > 0:
        increase = stepsize - mod
        file = open(filename,'a')         # append
#        for i in range (0, increase):
#            file.write(' ')
        file.write(' '*increase)
        file.close()

def RoundupString(data, stepsize):
    size = len(data)
    mod = size % stepsize
    increase = 0
    addon = ''
    if mod > 0:
        increase = stepsize-mod
#        for i in range(0,increase):
#            addon = addon + ' '
        addon = ' ' * increase
    return data + addon

def AddNL(s):
    return s + "\n"

def Data():
    return "Data"

def Parity():
    return "Parity"

# Have had some troubles with jelly/banana
# Plan to move to my own serialization of objects but leaving this here for now
def BinaryToAscii(input):
    return base64.encodestring(input)

def AsciiToBinary(input):
    return base64.decodestring(input)

def ObjectToString(input):
    banana.setPrefixLimit(200000000)                              # 200 MB is ok - does not work
    banana.SIZE_LIMIT = 200000000                                 # PREPRO not sure this is nice
    return banana.encode(jelly.jelly(input))

def StringToObject(input):
    banana.setPrefixLimit(200000000)                              # 200 MB is ok
    banana.SIZE_LIMIT = 200000000                                 # PREPRO not sure this is nice
    if len(input) == 0:
        dhnio.Dprint(1, 'misc.StringToObject ERROR in banana.decode, data length=0')
        return None
    try:
        bananaDecode = banana.decode(input)
    except:
        if len(input) > 0:
            fd, filename = tmpfile.make('other', '', 'banana.error-')
            os.write(fd, input)
            os.close(fd)
        dhnio.Dprint(1, 'misc.StringToObject ERROR in banana.decode, data length=%d' % len(input))
        dhnio.DprintException()
        return None

    try:
        # works for 384 bit RSA keys
        unjellyObject = jelly.unjelly(bananaDecode)
    except:
        # small patch to fix new dhnpacket location
        # this is for oldest code to understand newest code
        if bananaDecode[0] in [
                'lib.dhnpacket.dhnpacket',
                'p2p.datahaven.lib.dhnpacket.dhnpacket',
                ]:
            bananaDecode[0] = 'datahaven.lib.dhnpacket.dhnpacket'

        # different versions need to understand each other
        # this is for newest code to understand oldest code
        elif bananaDecode[0] == 'datahaven.lib.dhnpacket.dhnpacket':
            bananaDecode[0] = 'lib.dhnpacket.dhnpacket'

        # try again
        try:
            unjellyObject = jelly.unjelly(bananaDecode)
            dhnio.Dprint(4, 'misc.StringToObject bananaDecode[0]=%s' % str(bananaDecode[0]))
        except:
            if len(bananaDecode) > 0:
                fd, filename = tmpfile.make('other', '', 'jelly.error-')
                os.write(fd, input)
                os.close(fd)
            dhnio.Dprint(1, 'misc.StringToObject ERROR in jelly.unjelly, data length=%d' % len(str(input)))
            dhnio.DprintException()
            return None

    return unjellyObject

def ObjectToAscii(input):
    return BinaryToAscii(ObjectToString(input))

def AsciiToObject(input):
    return StringToObject(AsciiToBinary(input))              # works for 384 bit RSA keys

def pack_url_param(s):
    try:
        return urllib.quote(s)
    except:
        try:
            return str(urllib.quote(str(s)))
        except:
            dhnio.DprintException()
    return s

def unpack_url_param(s, default=None):
    if s is None or s == '':
        if default is not None:
            return default
        return s
    try:
        return urllib.unquote(str(s))
    except:
        dhnio.DprintException()
        return default

def rndstr(length):
    return ''.join([random.choice(string.letters+string.digits) for i in range(0,length)])

def stringToLong(s):
    return long('\0'+s, 256)

def longToString(n):
    s = n.tostring()
    if s[0] == '\0' and s != '\0':
        s = s[1:]
    return s

def receiptIDstr(receipt_id):
    try:
        return '%08d' % int(receipt_id)
    except:
        return str(receipt_id)

def username2idurl(username, host='identity.datahaven.net'):
    return 'http://' + host + '/' + username + '.xml'

def calculate_best_dimension(sz, maxsize=8):
    w = int(round(math.sqrt(sz)))
    h = w
    if w * h < sz:
        w += 1
    if w > maxsize:
        w = maxsize
        h = int(round(sz / w))
    return (1 if w==0 else w), (1 if h==0 else h)

def calculate_padding(w, h):
    imgW = 64
    imgH = 64
    if w >= 4:
        imgW = 4 * imgW / w
        imgH = 4 * imgH / w
    padding = 64/w - 8
    return imgW, imgH, padding

def getDeltaTime(tm):
    try:
#        tm = time.mktime(time.strptime(self.backupID, "F%Y%m%d%I%M%S%p"))
        dt = round(time.time() - tm)
        if dt > 2*60*60:
            return round(dt/(60.0*60.0)), 'hours'
        if dt > 60:
            return round(dt/60.0), 'minutes'
        return dt, 'seconds'
    except:
        return None, None

def getRealHost(host, port=None):
    if isinstance(host, str):
        if host.startswith('http://'):
            host = host[7:]
        elif host.startswith('cspace://'):
            host = host[9:]
        if port is not None:
            host += ':' + str(port)
    elif isinstance(host, tuple) and len(host) == 2:
        host = host[0] + ':' + str(host[1])
    elif host is None:
        host = 'None'
    else:
        if getattr(host, 'host', None) is not None:
            if getattr(host, 'port', None) is not None:
                host = str(getattr(host, 'host')) + ':' + str(getattr(host, 'port'))
            else:
                host = str(getattr(host, 'host'))
        elif getattr(host, 'underlying', None) is not None:
            host = str(getattr(host, 'underlying'))
        else:
            host = str(host)
            if port is not None:
                host += ':' + str(port)
    return host


#split strings "%dx%d+%d+%d" into 4 integers
def split_geom_string(geomstr):
    try:
        r = re.split('\D+', geomstr, 4)
        return int(r[0]), int(r[1]), int(r[2]), int(r[3])
    except:
        return None, None, None, None


def percent2string(percent, precis=3):
    s = str(round(percent, precis))
    if s[-2:] == '.0':
        s = s[:-2]
    return s + '%'

#def FloatToString(input):
#    formatted = '%.10f' % (input)
#    stripped = re.split('0+$', formatted)
#    if stripped[0][-1] in ['.', ',']: # some localizations use ',' instead of '.'
#        return stripped[0] + '0'
#    else:
#        return stripped[0]
def float2str(float_value, mask='%6.8f', no_trailing_zeros=True):
    try:
        f = float(float_value)
    except:
        return float_value 
    s = mask % f
    if no_trailing_zeros:
        s = s.rstrip('0').rstrip('.')
    return s

def seconds_to_time_left_string(seconds):
    s = int(seconds)
    years = s // 31104000
    if years > 1:
        return '%d years' % years
    s = s - (years * 31104000)
    months = s // 2592000
    if years == 1:
        r = 'one year'
        if months > 0:
            r += ' and %d months' % months
        return r
    if months > 1:
        return '%d months' % months
    s = s - (months * 2592000)
    days = s // 86400
    if months == 1:
        r = 'one month'
        if days > 0:
            r += ' and %d days' % days
        return r
    if days > 1:
        return '%d days' % days
    s = s - (days * 86400)
    hours = s // 3600
    if days == 1:
        r = 'one day'
        if hours > 0:
            r += ' and %d hours' % hours
        return r 
    s = s - (hours * 3600)
    minutes = s // 60
    seconds = s - (minutes * 60)
    if hours >= 6:
        return '%d hours' % hours
    if hours >= 1:
        r = '%d hours' % hours
        if hours == 1:
            r = 'one hour'
        if minutes > 0:
            r += ' and %d minutes' % minutes
        return r
    if minutes == 1:
        r = 'one minute'
        if seconds > 0:
            r += ' and %d seconds' % seconds
        return r
    if minutes == 0:
        return '%d seconds' % seconds
    if seconds == 0:
        return '%d minutes' % minutes
    return '%d minutes and %d seconds' % (minutes, seconds)

#------------------------------------------------------------------------------ 

def unicode_to_str_safe(unicode_string, encodings=None):
    try:
        return str(unicodestr) # .decode('utf-8')
    except:
        pass
    if encodings is None:
        encodings = [locale.getpreferredencoding(),] #  'utf-8' 
    output = ''
    for i in range(len(unicode_string)):
        unicode_char = unicode_string[i]
        char = '?'
        try:
            char = unicode_char.encode(encodings[0])
            # print char, encodings[0]
        except:
            for encoding in encodings:
                try:
                    char = unicode_char.encode(encoding)
                    # print char, encoding
                    break
                except:
                    pass
        output += char
    return output

#------------------------------------------------------------------------------

def getClipboardText():
    if dhnio.Windows():
        try:
            import win32clipboard
            import win32con
            win32clipboard.OpenClipboard()
            d = win32clipboard.GetClipboardData(win32con.CF_TEXT)
            win32clipboard.CloseClipboard()
            return d.replace('\r\n','\n')
        except:
            dhnio.DprintException()
            return ''
    elif dhnio.Linux():
        try:
            import wx
            # may crash, otherwise
            # this needs app.MainLoop() to be started 
            if not wx.TheClipboard.IsOpened():  
                do = wx.TextDataObject()
                wx.TheClipboard.Open()
                success = wx.TheClipboard.GetData(do)
                wx.TheClipboard.Close()
                if success:
                    return do.GetText()
                else:
                    return ''
            else:
                return ''
        except:
            return ''
    else:
        return ''

def setClipboardText(txt):
    if dhnio.Windows():
        try:
            import win32clipboard
            import win32con
            win32clipboard.OpenClipboard()
            win32clipboard.EmptyClipboard()
            win32clipboard.SetClipboardData(win32con.CF_TEXT, txt)
            win32clipboard.CloseClipboard()
        except:
            dhnio.DprintException()
    elif dhnio.Linux():
        try:
            import wx
            clipdata = wx.TextDataObject()
            clipdata.SetText(txt)
            if wx.TheClipboard:
                wx.TheClipboard.Open()
                wx.TheClipboard.SetData(clipdata)
                wx.TheClipboard.Close()
        except:
            dhnio.DprintException()
    else:
        #TODO
        return
              
#------------------------------------------------------------------------------ 

def isValidTransport(transport):
    global validTransports
    if transport in validTransports:
        return True
    else:
        return False

def validateTransports(orderL):
    global validTransports
    transports = []
    for transport in orderL:
        if isValidTransport(transport):
            transports.append(transport)
        else:
            dhnio.Dprint(6, 'misc.validateTransports WARNING invalid entry int transport list: %s , ignored' % str(transport))
    if len(transports) == 0:
        dhnio.Dprint(1, 'misc.validateTransports ERROR no valid transports, using default transports ' + str(validTransports))
        transports = validTransports
#    if len(transports) != len(orderL):
#        dhnio.Dprint(1, 'misc.validateTransports ERROR Transports contained an invalid entry, need to figure out where it came from.')
    return transports

def setTransportOrder(orderL):
    orderl = orderL
    orderL = validateTransports(orderL)
    orderTxt = string.join(orderl, ' ')
    dhnio.Dprint(8, 'misc.setTransportOrder: ' + str(orderTxt))
    dhnio.WriteFile(settings.DefaultTransportOrderFilename(), orderTxt)

def getTransportOrder():
    global validTransports
    dhnio.Dprint(8, 'misc.getTransportOrder')
    order = dhnio.ReadTextFile(settings.DefaultTransportOrderFilename()).strip()
    if order == '':
        orderL = validTransports
    else:
        orderL = order.split(' ')
        orderL = validateTransports(orderL)
    setTransportOrder(orderL)
    return orderL

def getOrderFromContacts(ident):
    return ident.getProtoOrder()

#-------------------------------------------------------------------------------

def StartWebStream():
    dhnio.Dprint(6,"misc.StartWebStream")
    import weblog
    weblog.init(settings.getWebStreamPort())
    dhnio.SetWebStream(weblog.log)

def StopWebStream():
    dhnio.Dprint(6,"misc.StopWebStream")
    dhnio.SetWebStream(None)
    import weblog
    weblog.shutdown()

def StartWebTraffic(root=None, path='traffic'):
    dhnio.Dprint(6,"misc.StartWebStream")
    import lib.webtraffic as webtraffic
    webtraffic.init(root, path, settings.getWebTrafficPort())

def StopWebTraffic():
    dhnio.Dprint(6,"misc.StopWebTraffic")
    import lib.webtraffic as webtraffic
    webtraffic.shutdown()

#------------------------------------------------------------------------------

def hmac_hash(string):
    h = hmac.HMAC(settings.HMAC_key_word(), string)
    return h.hexdigest().upper()

def encode64(s):
    return base64.b64encode(s)

def decode64(s):
    return base64.b64decode(s)

def get_hash(src):
##    h = md5.new(src).hexdigest()
    return hashlib.md5(src).hexdigest()

def file_hash(path):
    src = dhnio.ReadBinaryFile(path)
    if not src:
        return None
    return get_hash(src)

#-------------------------------------------------------------------------------

def time2daystring(tm=None):
    tm_ = tm
    if tm_ is None:
        tm_ = time.time()
    return time.strftime('%Y%m%d', time.localtime(tm_))

def daystring2time(daystring):
    try:
        t = time.strptime(daystring, '%Y%m%d')
    except:
        return None
    return time.mktime(t)

def time2str(format):
    return time.strftime(format)

def gmtime2str(format, seconds=None):
    if not seconds:
        return time.strftime(format, time.gmtime())
    return time.strftime(format, time.gmtime(seconds))

def str2gmtime(time_string, format):
    return time.mktime(time.strptime(time_string, format))

#------------------------------------------------------------------------------ 

def ReadRepoLocation():
    if dhnio.Linux():
        repo_file = os.path.join(dhnio.getExecutableDir(), 'repo.txt')
        if os.path.isfile(repo_file):
            src = dhnio.ReadTextFile(repo_file)
            if src:
                try:
                    return src.split('\n')[0].strip(), src.split('\n')[1].strip() 
                except:
                    dhnio.DprintException()
        return 'sources', 'http://datahaven.net/download.html'
            
        #TODO need to add something - so user can change the current repository
#        if dhnio.getExecutableDir().count('/usr/share/datahaven'):
#            return 'devel', 'http://datahaven.net/apt'
#        return 'sources', 'http://datahaven.net/download.html'
    src = dhnio.ReadTextFile(settings.RepoFile()).strip()
    if src == '':
        return settings.DefaultRepo(), settings.UpdateLocationURL(settings.DefaultRepo())
    l = src.split('\n')
    if len(l) < 2:
        return settings.DefaultRepo(), settings.UpdateLocationURL(settings.DefaultRepo())
    return l[0], l[1]   

#-------------------------------------------------------------------------------

def SetAutorunWindows():
    if os.path.abspath(dhnio.getExecutableDir()) != os.path.abspath(settings.WindowsBinDir()):
        return
    createWindowsShortcut(
        'DataHaven.NET.lnk',
        '%s' % settings.getIconLaunchFilename(),
        dhnio.getExecutableDir(),
        os.path.join(dhnio.getExecutableDir(), 'icons', settings.IconFilename()),
        '',
        'Startup', )

def ClearAutorunWindows():
    removeWindowsShortcut('DataHaven.NET.lnk', folder='Startup')

#def SetAutorunWindowsOld(CUorLM='CU', location=settings.getAutorunFilename(), name=settings.ApplicationName()):
#    cmdexec = r'reg add HK%s\software\microsoft\windows\currentversion\run /v "%s" /t REG_SZ /d "%s" /f' % (CUorLM, name, location)
#    dhnio.Dprint(6, 'misc.SetAutorunWindows executing: ' + cmdexec)
#    return nonblocking.ExecuteString(cmdexec)
#
#def ClearAutorunWindowsOld(CUorLM='CU', name = settings.ApplicationName()):
#    cmdexec = r'reg delete HK%s\software\microsoft\windows\currentversion\run /v "%s" /f' % (CUorLM, name)
#    dhnio.Dprint(6, 'misc.ClearAutorunWindows executing: ' + cmdexec)
#    return nonblocking.ExecuteString(cmdexec)

#-------------------------------------------------------------------------------

def transport_control_remove_after():
    global _RemoveAfterSent
    return _RemoveAfterSent

def set_transport_control_remove_after(flag):
    global _RemoveAfterSent
    _RemoveAfterSent = flag

#-------------------------------------------------------------------------------

def pathToWindowsShortcut(filename, folder='Desktop'):
    try:
        from win32com.client import Dispatch
        shell = Dispatch('WScript.Shell')
        desktop = shell.SpecialFolders(folder)
        return os.path.join(desktop, filename)
    except:
        dhnio.DprintException()
        return ''

def createWindowsShortcut(filename, target='', wDir='', icon='', args='', folder='Desktop'):
    if dhnio.Windows():
        try:
            from win32com.client import Dispatch
            shell = Dispatch('WScript.Shell')
            desktop = shell.SpecialFolders(folder)
            path = os.path.join(desktop, filename)
            shortcut = shell.CreateShortCut(path)
            shortcut.Targetpath = target
            shortcut.WorkingDirectory = wDir
            shortcut.Arguments = args
            if icon != '':
                shortcut.IconLocation = icon
            shortcut.save()
        except:
            dhnio.DprintException()

def removeWindowsShortcut(filename, folder='Desktop'):
    if dhnio.Windows():
        path = pathToWindowsShortcut(filename, folder)
        if os.path.isfile(path) and os.access(path, os.W_OK):
            try:
                os.remove(path)
            except:
                dhnio.DprintException()

#-------------------------------------------------------------------------------

def pathToStartMenuShortcut(filename):
    try:
        from win32com.shell import shell, shellcon
        from win32com.client import Dispatch
        shell_ = Dispatch('WScript.Shell')
        csidl = getattr(shellcon, 'CSIDL_PROGRAMS')
        startmenu = shell.SHGetSpecialFolderPath(0, csidl, False)
        return os.path.join(startmenu, filename)
    except:
        dhnio.DprintException()
        return ''

def createStartMenuShortcut(filename, target='', wDir='', icon='', args=''):
    if dhnio.Windows():
        try:
            from win32com.shell import shell, shellcon
            from win32com.client import Dispatch
            shell_ = Dispatch('WScript.Shell')
            csidl = getattr(shellcon, 'CSIDL_PROGRAMS')
            startmenu = shell.SHGetSpecialFolderPath(0, csidl, False)
            path = os.path.join(startmenu, filename)
            shortcut = shell_.CreateShortCut(path)
            shortcut.Targetpath = target
            shortcut.WorkingDirectory = wDir
            shortcut.Arguments = args
            if icon != '':
                shortcut.IconLocation = icon
            shortcut.save()
        except:
            dhnio.DprintException()

def removeStartMenuShortcut(filename):
    if dhnio.Windows():
        path = pathToStartMenuShortcut(filename)
        if os.path.isfile(path) and os.access(path, os.W_OK):
            try:
                os.remove(path)
            except:
                dhnio.DprintException()
    return

#------------------------------------------------------------------------------ 

def DoRestart(param=''):
    if dhnio.Windows():
        if dhnio.isFrozen():
            dhnio.Dprint(2, "misc.DoRestart under Windows (Frozen), param=%s" % param)
            dhnio.Dprint(2, "misc.DoRestart sys.executable=" + sys.executable)
            dhnio.Dprint(2, "misc.DoRestart sys.argv=" + str(sys.argv))
            starter_filepath = os.path.join(dhnio.getExecutableDir(), settings.WindowsStarterFileName())
            if not os.path.isfile(starter_filepath):
                dhnio.Dprint(2, "misc.DoRestart ERROR %s not found" % starter_filepath)
                return
            cmdargs = [os.path.basename(starter_filepath),]
            if param != '':
                cmdargs.append(param)
            dhnio.Dprint(2, "misc.DoRestart cmdargs="+str(cmdargs))
            os.spawnve(os.P_DETACH, starter_filepath, cmdargs, os.environ)

        else:
            dhnio.Dprint(2, "misc.DoRestart under Windows param=%s" % param)
            dhnio.Dprint(2, "misc.DoRestart sys.executable=" + sys.executable)
            dhnio.Dprint(2, "misc.DoRestart sys.argv=" + str(sys.argv))

            pypath = sys.executable
            cmdargs = [sys.executable]
            cmdargs.append(sys.argv[0])
            cmdargs += sys.argv[1:]
            if param != '' and not sys.argv.count(param):
                cmdargs.append(param)
            if cmdargs.count('restart'):
                cmdargs.remove('restart')

            dhnio.Dprint(2, "misc.DoRestart cmdargs="+str(cmdargs))
            # os.spawnve(os.P_DETACH, pypath, cmdargs, os.environ)
            os.execvpe(pypath, cmdargs, os.environ)

    else:
        dhnio.Dprint(2, "misc.DoRestart under Linux param=%s" % param)
        dhnio.Dprint(2, "misc.DoRestart sys.executable=" + sys.executable)
        dhnio.Dprint(2, "misc.DoRestart sys.argv=" + str(sys.argv))
        
        pypyth = sys.executable
        cmdargs = [sys.executable]
        if sys.argv[0] == '/usr/share/datahaven/dhn.py':
            cmdargs.append('/usr/bin/datahaven')
        else:
            cmdargs.append(sys.argv[0])
        if param:
            cmdargs.append(param)

        pid = os.fork()
        if pid == 0:
            dhnio.Dprint(2, "misc.DoRestart cmdargs="+str(cmdargs))
            os.execvpe(pypyth, cmdargs, os.environ)
        else:
            dhnio.Dprint(2, "misc.DoRestart os.fork returned "+str(pid))
            
            
def RunBatFile(filename, output_filename=None):
    if not dhnio.Windows():
        return
    dhnio.Dprint(0, 'misc.RunBatFile going to execute ' + str(filename))

    cmd = os.path.abspath(filename).replace('\\', '/')
    if output_filename is not None:
        cmd += ' >"%s"' % os.path.abspath(output_filename).replace('\\', '/')

    return RunShellCommand(cmd, False)

#    cmdargs = [filename, ]

#    # method 1
#    try:
#        import win32process
#        BatProcess = nonblocking.Popen(
#            cmdargs,
##            shell=True,
#            stdin=subprocess.PIPE,
##            stdout=stdout_file,
##            stderr=stdout_file,
#            creationflags = win32process.CREATE_NO_WINDOW,)
#    except:
#        dhnio.Dprint(1, 'misc.RunBatFile ERROR executing: ' + str(filename))
#        dhnio.DprintException()
#        BatProcess = None
#    return BatProcess

#    # method 2
#    filename = os.path.abspath(filename).replace('\\', '/')
#    cmdargs = [filename, ]
#    if output_filename is not None:
#        cmdargs.append('>"%s"' % output_filename)
#    dhnio.Dprint(0, 'misc.RunBatFile cmdargs=' + str(cmdargs))
#    return os.spawnve(os.P_DETACH, filename, cmdargs, os.environ)


def RunShellCommand(cmdstr, wait=True):
    dhnio.Dprint(8, 'misc.RunShellCommand ' + cmdstr)
    try:
        if dhnio.Windows():
            import win32process
            p = subprocess.Popen(
                cmdstr,
                shell=True,
                creationflags = win32process.CREATE_NO_WINDOW,)
        else:
            p = subprocess.Popen(
                cmdstr,
                shell=True,)
    except:
        dhnio.DprintException()
        return None
    if wait:
        try:
            result = p.wait()
        except:
            dhnio.DprintException()
            return None
    else:
        return p.pid
    return result


def ExplorePathInOS(filepath):
    try:
        if dhnio.Windows():
            # os.startfile(filepath)
            if os.path.isfile(filepath):
                subprocess.Popen(['explorer', '/select,', '%s' % (filepath.replace('/','\\'))])
            else:
                subprocess.Popen(['explorer', '%s' % (filepath.replace('/','\\'))])

        elif dhnio.Linux():
            subprocess.Popen(['xdg-open', filepath])

        elif dhnio.Mac():
            subprocess.Popen(['open', filepath])

    except:
        try:
            import webbrowser
            webbrowser.open(filepath)
        except:
            dhnio.DprintException()
    return


#def CopyBinariesAndUserDataOnDrive(drive):
#    #TODO
#    return
#
#    if not dhnio.Windows():
#        binSize = dhnio.getDirectorySize(dhnio.getExecutableDir())
#        dataSize = dhnio.getDirectorySize(settings.BaseDir())
#
#        cmdargs1 = ['cp', '-r', dhnio.getExecutableDir(), os.path.join(drive,'DataHaven.NET')]
#        dhnio.Dprint(4, 'misc.CopyBinariesAndUserDataOnDrive call '+str(cmdargs1))
#        subprocess.call(cmdargs1)
#        cmdargs2 = ['cp', '-r', settings.BaseDir(), os.path.join(drive,'datahavennet')]
#        dhnio.Dprint(4, 'misc.CopyBinariesAndUserDataOnDrive call '+str(cmdargs2))
#        subprocess.call(cmdargs2)
#        return 'ok'
#
#    else:
#        import win32api
#        binSize = dhnio.getDirectorySize(dhnio.getExecutableDir())
#        dataSize = dhnio.getDirectorySize(settings.BaseDir())
#        t = win32api.GetDiskFreeSpace(os.path.abspath(drive))
#        freeSpace = t[0] * t[1] * t[2]
#        if binSize + dataSize + 1024 * 1024 * 5 > freeSpace:
#            return 'space %d Mb' % ((binSize + dataSize) / (1024*1024))
#
#        dir1 = os.path.abspath(os.path.join(drive,'DataHaven.NET'))
#        dir2 = os.path.abspath(os.path.join(drive,'datahavennet'))
#
#        if os.path.exists(dir1):
#            return 'exist '+dir1
#
#        if os.path.exists(dir2):
#            return 'exist '+dir2
#
#        cmdstr1 = 'mkdir ' + dir1
#        cmdstr2 = 'mkdir ' + dir2
#        cmdstr3 = 'xcopy "%s" %s /E /K /R /H /Y' % (os.path.join(dhnio.getExecutableDir(), '*.*'), dir1)
#        cmdstr4 = 'xcopy "%s" %s /E /K /R /H /Y' % (os.path.join(settings.BaseDir(), '*.*'), dir2)
#
#        if RunShellCommand(cmdstr1) is None:
#            return 'error'
#        if RunShellCommand(cmdstr2) is None:
#            return 'error'
#        if RunShellCommand(cmdstr3) is None:
#            return 'error'
#        if RunShellCommand(cmdstr4) is None:
#            return 'error'
#
#        return 'ok'


def MoveFolderWithFiles(current_dir, new_dir, remove_old=False):
    if os.path.abspath(current_dir) == os.path.abspath(new_dir):
        return
    
    current = cmdLineQuote(current_dir)
    new = cmdLineQuote(new_dir)
    
    try:
        if dhnio.Linux():
            cmdargs = ['cp', '-r', current, new]
            dhnio.Dprint(4, 'misc.MoveFolderWithFiles wish to call: ' + str(cmdargs))
            subprocess.call(cmdargs)
            if remove_old:
                cmdargs = ['rm', '-r', current]
                dhnio.Dprint(4, 'misc.MoveFolderWithFiles wish to call: ' + str(cmdargs))
                subprocess.call(cmdargs)
            return 'ok'
    
        if dhnio.Windows():
            cmdstr0 = 'mkdir %s' % new
            cmdstr1 = 'xcopy %s %s /E /K /R /H /Y' % (cmdLineQuote(os.path.join(current_dir, '*.*')), new)
            cmdstr2 = 'rmdir /S /Q %s' % current
            if not os.path.isdir(new):
                dhnio.Dprint(4, 'misc.MoveFolderWithFiles wish to call: ' + str(cmdstr0))
                if RunShellCommand(cmdstr0) is None:
                    return 'error'
                dhnio.Dprint(4, 'misc.MoveFolderWithFiles wish to call: ' + str(cmdstr1))
            if RunShellCommand(cmdstr1) is None:
                return 'error'
            if remove_old:
                dhnio.Dprint(4, 'misc.MoveFolderWithFiles wish to call: ' + str(cmdstr2))
                if RunShellCommand(cmdstr2) is None:
                    return 'error'
            return 'ok'
        
    except:
        return 'failed'
    
    return 'ok'

#------------------------------------------------------------------------------

def UpdateDesktopShortcut():
    dhnio.Dprint(6, 'misc.UpdateDesktopShortcut')
    if dhnio.Windows() and dhnio.isFrozen():
        if settings.getGeneralDesktopShortcut():
            if not os.path.exists(pathToWindowsShortcut(settings.getIconLinkFilename())):
                createWindowsShortcut(
                    settings.getIconLinkFilename(),
                    '%s' % settings.getIconLaunchFilename(),
                    dhnio.getExecutableDir(),
                    os.path.join(dhnio.getExecutableDir(), 'icons', settings.IconFilename()),
                    'show' )
        else:
            if os.path.exists(pathToWindowsShortcut(settings.getIconLinkFilename())):
                removeWindowsShortcut(settings.getIconLinkFilename())

    # under Linux we need to mark file to be executable.
    # let's disable the desktop icon for now
    # elif dhnio.Linux():
        # if settings.getGeneralDesktopShortcut():
        #     RunShellCommand("xdg-desktop-icon install --novendor datahaven.desktop")
        # else:
        #     RunShellCommand("xdg-desktop-icon uninstall datahaven.desktop")


def UpdateStartMenuShortcut():
    dhnio.Dprint(6, 'misc.UpdateStartMenuShortcut')
    if dhnio.Windows() and dhnio.isFrozen():
        if settings.getGeneralStartMenuShortcut():
            if not os.path.exists(pathToStartMenuShortcut(settings.getIconLinkFilename())):
                createStartMenuShortcut(
                    settings.getIconLinkFilename(),
                    '%s' % settings.getIconLaunchFilename(),
                    dhnio.getExecutableDir(),
                    os.path.join(dhnio.getExecutableDir(), 'icons', settings.IconFilename()),
                    'show' )
        else:
            if os.path.exists(pathToStartMenuShortcut(settings.getIconLinkFilename())):
                removeStartMenuShortcut('Data Haven .NET.lnk')

    # we do not need this at all
    # .deb file will take care of the application menu shortcut
    # elif dhnio.Linux():
    #     if settings.getGeneralStartMenuShortcut():
    #         RunShellCommand("xdg-desktop-menu install --novendor datahavenmenu.desktop")
    #     else:
    #         RunShellCommand("xdg-desktop-menu uninstall datahavenmenu.desktop")

def UpdateSettings():
    dhnio.Dprint(6, 'misc.UpdateSettings')

    # setting up a Desktop shortcuts
    # UpdateDesktopShortcut()
    # UpdateStartMenuShortcut()

    # setting up autorun
    # I just realized this.
    # we can not do this on Linux
    # we need permissions to write to the /etc/crontab
    # under Linux we can do it only during installing of datahaven package
    # Best choice is removing Autorun option at all!
    # So let's just check it every time we start (for Windows)
    # if dhnio.isFrozen() and dhnio.Windows():
    #     SetAutorunWindows()
    #     UpdateRegistryUninstall()
    # UPDATED: 
    # did a new setup installer for Windows.
    # it will handle all those things

#-------------------------------------------------------------------------------

def SendDevReportOld(subject, body, includelogs):
    try:
        filesList = []
        if includelogs:
            filesList.append(settings.LocalIdentityFilename())
            filesList.append(settings.UserConfigFilename())
            filesList.append(os.path.join(dhnio.getExecutableDir(), 'dhnmain.exe.log'))
            filesList.append(os.path.join(dhnio.getExecutableDir(), 'dhnmain.log'))
            for filename in os.listdir(settings.LogsDir()):
                filepath = os.path.join(settings.LogsDir(), filename)
                filesList.append(filepath)
        if len(filesList) > 0:
            import zipfile
            import tempfile
            import time
            username = dhnio.ReadTextFile(settings.UserNameFilename())
            zipfd, zipfilename = tempfile.mkstemp('.zip', username+'-'+time.strftime('%Y%m%d%I%M%S')+'-' )
            zfile = zipfile.ZipFile(zipfilename, "w", compression=zipfile.ZIP_DEFLATED)
            for filename in filesList:
                if os.path.isfile(filename):
                    try:
                        if os.path.getsize(filename) < 1024*512:
                            zfile.write(filename, os.path.basename(filename))
                    except:
                        pass
            zfile.close()
            filesList = [zipfilename]    
        dhnnet.SendEmail(
            'to@mail',
            'from@mail',
            'smtp.gmail.com',
            587,
            'user',
            'password',
            subject,
            body,
            filesList,)
    except:
        dhnio.DprintException()
        

def SendDevReport(subject, body, includelogs, progress=None, receiverDeferred=None):
    try:
        zipfilename = ''
        filesList = []
        if includelogs:
            filesList.append(settings.LocalIdentityFilename())
            filesList.append(settings.UserConfigFilename())
            filesList.append(os.path.join(dhnio.getExecutableDir(), 'dhnmain.exe.log'))
            filesList.append(os.path.join(dhnio.getExecutableDir(), 'dhnmain.log'))
            for filename in os.listdir(settings.LogsDir()):
                filepath = os.path.join(settings.LogsDir(), filename)
                filesList.append(filepath)
        if len(filesList) > 0:
            import zipfile
            import tempfile
            import time
            username = dhnio.ReadTextFile(settings.UserNameFilename())
            zipfd, zipfilename = tempfile.mkstemp('.zip', username+'-'+time.strftime('%Y%m%d%I%M%S')+'-' )
            zfile = zipfile.ZipFile(zipfilename, "w", compression=zipfile.ZIP_DEFLATED)
            for filename in filesList:
                if os.path.isfile(filename):
                    try:
                        if os.path.getsize(filename) < 1024*512:
                            zfile.write(filename, os.path.basename(filename))
                    except:
                        pass
            zfile.close()
        files = {}
        if zipfilename:
            files['upload'] = zipfilename
        data = {'subject': subject, 'body': body}
        dhnnet.uploadHTTP('http://datahaven.net/cgi-bin/feedback.py', files, data, progress, receiverDeferred)            
        return True 
    except:
        dhnio.DprintException()
    return False

#------------------------------------------------------------------------------ 

def GetUserProfilePicturePath():
    if dhnio.Windows():
        # http://social.msdn.microsoft.com/Forums/en/vcgeneral/thread/8c72b948-d32c-4785-930e-0d6fdf032ecc
        username = os.path.basename(os.path.expanduser('~'))
        if dhnio.windows_version() == 5: # Windows XP
            # %ALLUSERSPROFILE%\Application Data\Microsoft\User Account Pictures
            allusers = os.environ.get('ALLUSERSPROFILE', os.path.join(os.path.dirname(os.path.expanduser('~')), 'All Users'))
            return os.path.join(allusers, 'Application Data', 'Microsoft', 'User Account Pictures', username+'.bmp')
        elif dhnio.windows_version() == 6: # Windows 7
            # C:\Users\<username>\AppData\Local\Temp\<domain>+<username>.bmp 
            default_path = os.path.join(os.path.expanduser('~'), 'Application Data')
            return os.path.join(os.environ.get('APPDATA', default_path), 'Local', 'Temp', username+'.bmp')
        else:
            return ''
    elif dhnio.Linux():
        return os.path.join(os.path.expanduser('~'), '.face')
    return ''

def UpdateRegistryUninstall(uninstall=False):
    try:
        import _winreg
    except:
        return False
    
    unistallpath = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall" 
    regpath = unistallpath + "\\DataHaven.NET"
    values = {
        'DisplayIcon':      '%s,0' % str(dhnio.getExecutableFilename()),
        'DisplayName':      'DataHaven.NET',
        'DisplayVersion':   'rev'+dhnio.ReadTextFile(settings.RevisionNumberFile()).strip(),
        'InstallLocation:': settings.BaseDir(),
        'NoModify':         1,
        'NoRepair':         1,
        'UninstallString':  '%s uninstall' % dhnio.getExecutableFilename(),
        'URLInfoAbout':     'http://datahaven.net', }    

    # open
    try:
        reg = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, regpath, 0, _winreg.KEY_ALL_ACCESS)
    except:
        try:
            reg = _winreg.CreateKey(_winreg.HKEY_LOCAL_MACHINE, regpath)
        except:
            dhnio.DprintException()
            return False

    # check
    i = 0
    while True:
        try:
            name, value, typ = _winreg.EnumValue(reg, i)
        except:
            break
        i += 1
        if uninstall:
            try:
                _winreg.DeleteKey(reg, name)
            except:
                try:
                    _winreg.DeleteValue(reg, name)
                except:
                    dhnio.DprintException()
        else:
            if name == 'DisplayName' and value == 'DataHaven.NET':
                _winreg.CloseKey(reg)
                return True

    # delete
    if uninstall:
        _winreg.CloseKey(reg)
        try:
            reg = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, unistallpath, 0, _winreg.KEY_ALL_ACCESS)
        except:
            dhnio.DprintException()
            return False
        try:
            _winreg.DeleteKey(reg, 'DataHaven.NET')
        except:
            dhnio.DprintException()
            _winreg.CloseKey(reg)
            return False
        _winreg.CloseKey(reg)
        return True

    # write
    for key, value in values.items():
        typ = _winreg.REG_SZ
        if isinstance(value, int):
            typ = _winreg.REG_DWORD
        _winreg.SetValueEx(reg, key, 0, typ, value)
    _winreg.CloseKey(reg)

    return True


def MakeBatFileToUninstall(wait_appname='dhnmain.exe', local_dir=dhnio.getExecutableDir(), dirs2delete=[settings.BaseDir(),]):
    dhnio.Dprint(0, 'misc.MakeBatFileToUninstall')
    import tempfile
    batfileno, batfilename = tempfile.mkstemp('.bat', 'DataHaven.NET-uninstall-')
    batsrc = ''
    batsrc += 'cd "%s"\n' % local_dir
    batsrc += 'del search.log /Q\n'
    batsrc += ':again\n'
    batsrc += 'sleep 1\n'
    batsrc += 'tasklist /FI "IMAGENAME eq %s" /FO CSV > search.log\n' % wait_appname
    batsrc += 'FOR /F %%A IN (search.log) DO IF %%-zA EQU 0 GOTO again\n'
#    batsrc += 'start /B /D"%s" %s\n' % (local_dir, start_cmd)
#    batsrc += 'start /B /D"%s" %s\n' % (dhnio.shortPath(local_dir), start_cmd)
    for dirpath in dirs2delete:
        batsrc += 'rmdir /S /Q "%s"\n' % os.path.abspath(dirpath)
    batsrc += 'del /F /S /Q "%s"\n' % os.path.abspath(batfilename)
    print batsrc
    os.write(batfileno, batsrc)
    os.close(batfileno)
    return os.path.abspath(batfilename)

#------------------------------------------------------------------------------

def LoopAttenuation(current_delay, faster, min, max):
    global _AttenuationFactor
    if faster:
        return min
    else:
        if current_delay < max:
            return current_delay * _AttenuationFactor   
    return current_delay

#------------------------------------------------------------------------------ 

if __name__ == '__main__':
    dhnio.SetDebug(10)
    dhnio.init()
    init()

#    print time.localtime()    
#    v = NewBackupID()
#    i = v.rfind('M')
#    print v, i
#    ampm = v[i-1:i+1]
#    st_time = v[1:i-1]
#    print ampm, st_time
#    bk1 = '0/0/1/'+v+'2'
#    bk2 = '0/0/1/'+v+'1'
#    print backup_id_compare(bk1, bk2)
    
    # import base64
    # base64.
    
    # print len(BASE_ALPHABET)
    
    # a = FilePathToBackupID(sys.argv[1])
    # b = FilePathToBackupID(sys.argv[1], False)
    # b = BackupIDToFilePath(a)
    # print len(a), a
    # print b, len(b)
    # print os.path.exists(b)
    
    #debug
#    from twisted.internet import reactor
#    reactor.callLater(1, SetAutorun)
#    reactor.run()

#    target = 'C:\Program Files\DataHaven.Net\dhnmain.exe'
#    workdir = 'C:\Program Files\DataHaven.Net'


#    SetAutorun()

#    print calculate_best_dimension(2)

#    path = 'c:\\Program Files\\\xc4 \xd8 \xcd'
#    batfilename = MakeBatFile2Restart(local_dir=path)
##    RunBatFile(batfilename, settings.RunUpdateLogFilename())
#    RunBatFile(batfilename)

    #import imp
    #sys.modules['datahaven.lib.dhnpacket'] = imp.new_module('datahaven.lib.dhnpacket')
    #sys.modules['datahaven.lib.dhnpacket'].dhnpacket = dhnpacket.dhnpacket
    #.dhnpacket = dhnpacket.dhnpacket # sys.modules['dhnpacket']

    #m = sys.modules.keys()
    #m.sort()
    #print '\n'.join(m)

    #sss = open('jelly.error1').read()
    #o = StringToObject(sss)
    #print type(o), o

#    from twisted.internet.defer import setDebugging
#    setDebugging(True)

#    if True:
#        from twisted.internet import reactor
#        from twisted.internet.defer import Deferred        
#        def _progress(x, y):
#            print '%d/%d' % (x,y)      
#        def _done(x):
#            print 'DONE', x
#            reactor.stop()  
#        d = Deferred()
#        d.addBoth(_done)
#        SendDevReport('subject ' + time.strftime('%m:%s'), 'some body112', True, _progress, d)
#        reactor.run()
#    else:
#        SendDevReportOld('subject', 'some body\n LALALALAL\n DataHaven.NET for all!!!', True)

#    for i in range(24):
#        #print '-------------------'
#        tm = time.time()+i*60*60
#        #print tm
#        st = time.localtime(tm)
#        #print st
#        bid = NewBackupID(st)
#        #print bid
#        tm2 = TimeFromBackupID(bid)
#        #print tm2
#        ok = int(tm) == int(tm2)
#        print ok

#    for i in range(10):
#        print pow(8, i), seconds_to_time_left_string(pow(8, i))

